"""Error codes - copied from the PS6000 programmer's manual."""

# To get formatting correct do following copy-replace in
# Programmers Notepad
# 1. Copy/replace ' - ' with '", "'
# 2. Copy/replace '\r' with '"],\r' (enable slash expressions when doing)
# 3. Copy/replace '^([0-9A-F]{2} ){1}' with '0x\1, "' (w/ regex)
# 4. Copy/replace '^([0-9A-F]{3} ){1}' with '0x\1, "' (w/ regex)
# 5. Copy/repplace '0x' with '[0x'
ERROR_CODES = [
    [0x00, "PICO_OK", "The PicoScope XXXX is functioning correctly."],
    [0x01, "PICO_MAX_UNITS_OPENED",
     "An attempt has been made to open more than PSXXXX_MAX_UNITS."],
    [0x02, "PICO_MEMORY_FAIL",
     "Not enough memory could be allocated on the host machine."],
    [0x03, "PICO_NOT_FOUND", "No PicoScope XXXX could be found."],
    [0x04, "PICO_FW_FAIL", "Unable to download firmware."],
    [0x05, "PICO_OPEN_OPERATION_IN_PROGRESS"],
    [0x06, "PICO_OPERATION_FAILED"],
    [0x07, "PICO_NOT_RESPONDING",
     "The PicoScope XXXX is not responding to commands from the PC."],
    [0x08, "PICO_CONFIG_FAIL",
     "The configuration information in the PicoScope XXXX has become " +
     "corrupt or is missing."],
    [0x09, "PICO_KERNEL_DRIVER_TOO_OLD",
     "The picopp.sys file is too old to be used with the device driver."],
    [0x0A, "PICO_EEPROM_CORRUPT",
     "The EEPROM has become corrupt, so the device will use a default " +
     "setting."],
    [0x0B, "PICO_OS_NOT_SUPPORTED",
     "The operating system on the PC is not supported by this driver."],
    [0x0C, "PICO_INVALID_HANDLE",
     "There is no device with the handle value passed."],
    [0x0D, "PICO_INVALID_PARAMETER", "A parameter value is not valid."],
    [0x0E, "PICO_INVALID_TIMEBASE",
     "The timebase is not supported or is invalid."],
    [0x0F, "PICO_INVALID_VOLTAGE_RANGE",
     "The voltage range is not supported or is invalid."],
    [0x10, "PICO_INVALID_CHANNEL",
     "The channel number is not valid on this device or no channels have " +
     "been set."],
    [0x11, "PICO_INVALID_TRIGGER_CHANNEL",
     "The channel set for a trigger is not available on this device."],
    [0x12, "PICO_INVALID_CONDITION_CHANNEL",
     "The channel set for a condition is not available on this device."],
    [0x13, "PICO_NO_SIGNAL_GENERATOR",
     "The device does not have a signal generator."],
    [0x14, "PICO_STREAMING_FAILED",
     "Streaming has failed to start or has stopped without user request."],
    [0x15, "PICO_BLOCK_MODE_FAILED",
     "Block failed to start", "a parameter may have been set wrongly."],
    [0x16, "PICO_NULL_PARAMETER", "A parameter that was required is NULL."],
    [0x18, "PICO_DATA_NOT_AVAILABLE",
     "No data is available from a run block call."],
    [0x19, "PICO_STRING_BUFFER_TOO_SMALL",
     "The buffer passed for the information was too small."],
    [0x1A, "PICO_ETS_NOT_SUPPORTED", "ETS is not supported on this device."],
    [0x1B, "PICO_AUTO_TRIGGER_TIME_TOO_SHORT",
     "The auto trigger time is less than the time it will take to collect " +
     "the pre-trigger data."],
    [0x1C, "PICO_BUFFER_STALL",
     "The collection of data has stalled as unread data would be " +
     "overwritten."],
    [0x1D, "PICO_TOO_MANY_SAMPLES",
     "Number of samples requested is more than available in the current " +
     "memory segment."],
    [0x1E, "PICO_TOO_MANY_SEGMENTS",
     "Not possible to create number of segments requested."],
    [0x1F, "PICO_PULSE_WIDTH_QUALIFIER",
     "A null pointer has been passed in the trigger function or one of the " +
     "parameters is out of range."],
    [0x20, "PICO_DELAY",
     "One or more of the hold-off parameters are out of range."],
    [0x21, "PICO_SOURCE_DETAILS",
     "One or more of the source details are incorrect."],
    [0x22, "PICO_CONDITIONS", "One or more of the conditions are incorrect."],
    [0x23, "PICO_USER_CALLBACK",
     "The driver's thread is currently in the psXXXXBlockReady callback " +
     "function and therefore the action cannot be carried out."],
    [0x24, "PICO_DEVICE_SAMPLING",
     "An attempt is being made to get stored data while streaming. " +
     "Either stop streaming by calling psXXXXStop, or use " +
     "psXXXXGetStreamingLatestValues."],
    [0x25, "PICO_NO_SAMPLES_AVAILABLE",
     "because a run has not been completed."],
    [0x26, "PICO_SEGMENT_OUT_OF_RANGE",
     "The memory index is out of range."],
    [0x27, "PICO_BUSY", "Data cannot be returned yet."],
    [0x28, "PICO_STARTINDEX_INVALID",
     "The start time to get stored data is out of range."],
    [0x29, "PICO_INVALID_INFO",
     "The information number requested is not a valid number."],
    [0x2A, "PICO_INFO_UNAVAILABLE",
     "The handle is invalid so no information is available about the device." +
     " Only PICO_DRIVER_VERSION is available."],
    [0x2B, "PICO_INVALID_SAMPLE_INTERVAL",
     "The sample interval selected for streaming is out of range."],
    [0x2D, "PICO_MEMORY", "Driver cannot allocate memory."],
    [0x2E, "PICO_SIG_GEN_PARAM",
     "Incorrect parameter passed to signal generator."],
    [0x34, "PICO_WARNING_AUX_OUTPUT_CONFLICT",
     "AUX cannot be used as input and output at the same time."],
    [0x35, "PICO_SIGGEN_OUTPUT_OVER_VOLTAGE",
     "The combined peak to peak voltage and the analog offset voltage " +
     "exceed the allowable voltage the signal generator can produce."],
    [0x36, "PICO_DELAY_NULL", "NULL pointer passed as delay parameter."],
    [0x37, "PICO_INVALID_BUFFER",
     "The buffers for overview data have not been set while streaming."],
    [0x38, "PICO_SIGGEN_OFFSET_VOLTAGE",
     "The analog offset voltage is out of range."],
    [0x39, "PICO_SIGGEN_PK_TO_PK",
     "The analog peak to peak voltage is out of range."],
    [0x3A, "PICO_CANCELLED", "A block collection has been cancelled."],
    [0x3B, "PICO_SEGMENT_NOT_USED",
     "The segment index is not currently being used."],
    [0x3C, "PICO_INVALID_CALL",
     "The wrong GetValues function has been called for the collection mode " +
     "in use."],
    [0x3F, "PICO_NOT_USED", "The function is not available."],
    [0x40, "PICO_INVALID_SAMPLERATIO",
     "The aggregation ratio requested is out of range."],
    [0x41, "PICO_INVALID_STATE",
     "Device is in an invalid state."],
    [0x42, "PICO_NOT_ENOUGH_SEGMENTS",
     "The number of segments allocated is fewer than the number of captures " +
     "requested."],
    [0x43, "PICO_DRIVER_FUNCTION",
     "You called a driver function while another driver function was still " +
     "being processed."],
    [0x45, "PICO_INVALID_COUPLING",
     "An invalid coupling type was specified in psXXXXSetChannel."],
    [0x46, "PICO_BUFFERS_NOT_SET",
     "An attempt was made to get data before a data buffer was defined."],
    [0x47, "PICO_RATIO_MODE_NOT_SUPPORTED",
     "The selected downsampling mode (used for data reduction) is not " +
     "allowed."],
    [0x49, "PICO_INVALID_TRIGGER_PROPERTY",
     "An invalid parameter was passed to psXXXXSetTriggerChannelProperties."],
    [0x4A, "PICO_INTERFACE_NOT_CONNECTED",
     "The driver was unable to contact the oscilloscope."],
    [0x4D, "PICO_SIGGEN_WAVEFORM_SETUP_FAILED",
     "A problem occurred in psXXXXSetSigGenBuiltIn or " +
     "psXXXXSetSigGenArbitrary."],
    [0x4E, "PICO_FPGA_FAIL"],
    [0x4F, "PICO_POWER_MANAGER"],
    [0x50, "PICO_INVALID_ANALOGUE_OFFSET",
     "An impossible analogue offset value was specified in psXXXXSetChannel."],
    [0x51, "PICO_PLL_LOCK_FAILED",
     "Unable to configure the PicoScope XXXX."],
    [0x52, "PICO_ANALOG_BOARD",
     "The oscilloscope's analog board is not detected, or is not connected " +
     "to the digital board."],
    [0x53, "PICO_CONFIG_FAIL_AWG",
     "Unable to configure the signal generator."],
    [0x54, "PICO_INITIALISE_FPGA",
     "The FPGA cannot be initialized, so unit cannot be opened."],
    [0x56, "PICO_EXTERNAL_FREQUENCY_INVALID",
     "The frequency for the external clock is not within ±5% of the " +
     "stated value."],
    [0x57, "PICO_CLOCK_CHANGE_ERROR",
     "The FPGA could not lock the clock signal."],
    [0x58, "PICO_TRIGGER_AND_EXTERNAL_CLOCK_CLASH",
     "You are trying to configure the AUX input as both a trigger and a " +
     "reference clock."],
    [0x59, "PICO_PWQ_AND_EXTERNAL_CLOCK_CLASH",
     "You are trying to configure the AUX input as both a pulse width " +
     "qualifier and a reference clock."],
    [0x5A, "PICO_UNABLE_TO_OPEN_SCALING_FILE",
     "The scaling file set can not be opened."],
    [0x5B, "PICO_MEMORY_CLOCK_FREQUENCY",
     "The frequency of the memory is reporting incorrectly."],
    [0x5C, "PICO_I2C_NOT_RESPONDING",
     "The I2C that is being actioned is not responding to requests."],
    [0x5D, "PICO_NO_CAPTURES_AVAILABLE",
     "There are no captures available and therefore no data can be returned."],
    [0x5E, "PICO_NOT_USED_IN_THIS_CAPTURE_MODE",
     "The capture mode the device is currently running in does not support " +
     "the current request."],
    [0x103, "PICO_GET_DATA_ACTIVE", "Reserved"],
    [0x104, "PICO_IP_NETWORKED", "The device is currently connected via " +
     "the IP Network socket and thus the call made is not supported."],
    [0x105, "PICO_INVALID_IP_ADDRESS", "An IP address that is not correct " +
     "has been passed to the driver."],
    [0x106, "PICO_IPSOCKET_FAILED", "The IP socket has failed."],
    [0x107, "PICO_IPSOCKET_TIMEDOUT", "The IP socket has timed out."],
    [0x108, "PICO_SETTINGS_FAILED", "The settings requested have failed to " +
     "be set."],
    [0x109, "PICO_NETWORK_FAILED", "The network connection has failed."],
    [0x10A, "PICO_WS2_32_DLL_NOT_LOADED", "Unable to load the WS2 dll."],
    [0x10B, "PICO_INVALID_IP_PORT", "The IP port is invalid."],
    [0x10C, "PICO_COUPLING_NOT_SUPPORTED",
     "The type of coupling requested is not supported on the opened device."],
    [0x10D, "PICO_BANDWIDTH_NOT_SUPPORTED",
     "Bandwidth limit is not supported on the opened device."],
    [0x10E, "PICO_INVALID_BANDWIDTH",
     "The value requested for the bandwidth limit is out of range."],
    [0x10F, "PICO_AWG_NOT_SUPPORTED",
     "The device does not have an arbitrary waveform generator."],
    [0x110, "PICO_ETS_NOT_RUNNING",
     "Data has been requested with ETS mode set but run block has not been " +
     "called, or stop has been called."],
    [0x111, "PICO_SIG_GEN_WHITENOISE_NOT_SUPPORTED",
     "White noise is not supported on the opened device."],
    [0x112, "PICO_SIG_GEN_WAVETYPE_NOT_SUPPORTED",
     "The wave type requested is not supported by the opened device."],
    [0x116, "PICO_SIG_GEN_PRBS_NOT_SUPPORTED",
     "Siggen does not generate pseudorandom bit stream."],
    [0x117, "PICO_ETS_NOT_AVAILABLE_WITH_LOGIC_CHANNELS",
     "When a digital port is enabled, ETS sample mode is not available for " +
     "use."],
    [0x118, "PICO_WARNING_REPEAT_VALUE", "Not applicable to this device."],
    [0x119, "PICO_POWER_SUPPLY_CONNECTED",
     "The DC power supply is connected."],
    [0x11A, "PICO_POWER_SUPPLY_NOT_CONNECTED",
     "The DC power supply isn’t connected."],
    [0x11B, "PICO_POWER_SUPPLY_REQUEST_INVALID",
     "Incorrect power mode passed for current power source."],
    [0x11C, "PICO_POWER_SUPPLY_UNDERVOLTAGE",
     "The supply voltage from the USB source is too low."],
    [0x11D, "PICO_CAPTURING_DATA",
     "The device is currently busy capturing data."],
    [0x11E, "PICO_USB3_0_DEVICE_NON_USB3_0_PORT",
     "You must connect the device to a USB 3.0 port, or call " +
     "ps4000aChangePowerSource to switch the device into " +
     "non-USB 3.0-power mode"],
    [0x11F, "PICO_NOT_SUPPORTED_BY_THIS_DEVICE",
     "A function has been called that is not supported by the current " +
     "device variant."],
    [0x120, "PICO_INVALID_DEVICE_RESOLUTION",
     "The device resolution is invalid (out of range)."],
    [0x121, "PICO_INVALID_NUMBER_CHANNELS_FOR_RESOLUTION",
     "The number of channels which can be enabled is limited in " +
     "15 and 16-bit modes"],
    [0x122, "PICO_CHANNEL_DISABLED_DUE_TO_USB_POWERED",
     "USB Power not sufficient to power all channels."],
    [0x123, "PICO_SIGGEN_DC_VOLTAGE_NOT_CONFIGURABLE", ""],
    [0x124, "PICO_NO_TRIGGER_ENABLED_FOR_TRIGGER_IN_PRE_TRIG", ""],
    [0x125, "PICO_TRIGGER_WITHIN_PRE_TRIG_NOT_ARMED", ""],
    [0x126, "PICO_TRIGGER_WITHIN_PRE_NOT_ALLOWED_WITH_DELAY", ""],
    [0x127, "PICO_TRIGGER_INDEX_UNAVAILABLE", ""],
    [0x128, "PICO_AWG_CLOCK_FREQUENCY", ""],
    [0x129, "PICO_TOO_MANY_CHANNELS_IN_USE", ""],
    [0x12A, "PICO_NULL_CONDITIONS", ""],
    [0x12B, "PICO_DUPLICATE_CONDITION_SOURCE", ""],
    [0x12C, "PICO_INVALID_CONDITION_INFO", ""],
    [0x12D, "PICO_SETTINGS_READ_FAILED", ""],
    [0x12E, "PICO_SETTINGS_WRITE_FAILED", ""],
    [0x12F, "PICO_ARGUMENT_OUT_OF_RANGE", ""],
    [0x130, "PICO_HARDWARE_VERSION_NOT_SUPPORTED", ""],
    [0x131, "PICO_DIGITAL_HARDWARE_VERSION_NOT_SUPPORTED", ""],
    [0x132, "PICO_ANALOGUE_HARDWARE_VERSION_NOT_SUPPORTED", ""],
    [0x133, "PICO_UNABLE_TO_CONVERT_TO_RESISTANCE", ""],
    [0x134, "PICO_DUPLICATED_CHANNEL", ""],
    [0x135, "PICO_INVALID_RESISTANCE_CONVERSION", ""],
    [0x136, "PICO_INVALID_VALUE_IN_MAX_BUFFER", ""],
    [0x137, "PICO_INVALID_VALUE_IN_MIN_BUFFER", ""],
    [0x138, "PICO_SIGGEN_FREQUENCY_OUT_OF_RANGE", ""],
    [0x139, "PICO_EEPROM2_CORRUPT", ""],
    [0x13A, "PICO_EEPROM2_FAIL", ""],
    [0x13B, "PICO_SERIAL_BUFFER_TOO_SMALL", ""],
    [0x13C, "PICO_SIGGEN_TRIGGER_AND_EXTERNAL_CLOCK_CLASH", ""],
    [0x13D, "PICO_WARNING_SIGGEN_AUXIO_TRIGGER_DISABLED", ""],
    [0x13E, "PICO_SIGGEN_GATING_AUXIO_NOT_AVAILABLE", ""],
    [0x13F, "PICO_SIGGEN_GATING_AUXIO_ENABLED", ""],
    [0x140, "PICO_RESOURCE_ERROR", ""],
    [0x141, "PICO_TEMPERATURE_TYPE_INVALID", ""],
    [0x142, "PICO_TEMPERATURE_TYPE_NOT_SUPPORTED", ""],
    [0x143, "PICO_TIMEOUT", ""],
    [0x144, "PICO_DEVICE_NOT_FUNCTIONING", ""],
    [0x145, "PICO_INTERNAL_ERROR", ""],
    [0x146, "PICO_MULTIPLE_DEVICES_FOUND", ""],
    [0x147, "PICO_WARNING_NUMBER_OF_SEGMENTS_REDUCED", ""],
    [0x148, "PICO_CAL_PINS_STATES", ""],
    [0x149, "PICO_CAL_PINS_FREQUENCY", ""],
    [0x14A, "PICO_CAL_PINS_AMPLITUDE", ""],
    [0x14B, "PICO_CAL_PINS_WAVETYPE", ""],
    [0x14C, "PICO_CAL_PINS_OFFSET", ""],
    [0x14D, "PICO_PROBE_FAULT", ""],
    [0x14E, "PICO_PROBE_IDENTITY_UNKNOWN", ""],
    [0x14F, "PICO_PROBE_POWER_DC_POWER_SUPPLE_REQUIRED", ""],
    [0x150, "PICO_PROBE_NOT_POWERED_THROUGH_DC_POWER_SUPPLY", ""],
    [0x151, "PICO_PROBE_CONFIG_FAILURE", ""],
    [0x152, "PICO_PROBE_INTERACTION_CALLBACK", ""],
    [0x153, "PICO_UNKNOWN_INTELLIGENT_PROBE", ""],
    [0x154, "PICO_INTELLIGENT_PROBE_CORRUPT", ""],
    [0x155, "PICO_PROBE_COLLECTION_NOT_STARTED", ""],
    [0x156, "PICO_PROBE_POWER_CONSUMPTION_EXCEEDED", ""],
    [0x157, "PICO_WARNING_PROBE_CHANNEL_OUT_OF_SYNC", ""],
    [0x158, "PICO_ENDPOINT_MISSING", ""],
    [0x159, "PICO_UNKNOWN_ENDPOINT_REQUEST", ""],
    [0x15A, "PICO_ADC_TYPE_ERROR", ""],
    [0x15B, "PICO_FPGA2_FAILED", ""],
    [0x15C, "PICO_FPGA2_DEVICE_STATUS", ""],
    [0x15D, "PICO_ENABLED_PROGRAM_FPGA2_FAILED", ""],
    [0x15E, "PICO_NO_CANNELS_OR_PORTS_ENABLED", ""],
    [0x15F, "PICO_INVALID_RATIO_MODE", ""],
    [0x160, "PICO_READS_NOT_SUPPORTED_IN_CURRENT_CAPTURE_MODE", ""],
    [0x161, "PICO_TRIGGER_READ_SELECTION_CHECK_FAILED", ""],
    [0x162, "PICO_DATA_READ1_SELECTION_CHECK_FAILED", ""],
    [0x164, "PICO_DATA_READ2_SELECTION_CHECK_FAILED", ""],
    [0x168, "PICO_DATA_READ3_SELECTION_CHECK_FAILED", ""],
    [0x170, "PICO_READ_SELECTION_OUT_OF_RANGE", ""],
    [0x171, "PICO_MULTIPLE_RATIO_MODES", ""],
    [0x172, "PICO_NO_SAMPLES_READ", ""],
    [0x173, "PICO_RATIO_MODE_NOT_REQUESTED", ""],
    [0x174, "PICO_NO_USER_READ_REQUESTS", ""],
    [0x175, "PICO_ZERO_SAMPLES_INVALID", ""],
    [0x176, "PICO_ANALOGUE_HARDWARE_MISSING", ""],
    [0x177, "PICO_ANALOGUE_HARDWARE_PINS", ""],
    [0x178, "PICO_ANALOGUE_SMPS_FAULT", ""],
    [0x179, "PICO_DIGITAL_ANALOGUE_HARDWARE_CONFLICT", ""],
    [0x17A, "PICO_RATIO_MODE_BUFFER_NOT_SET", ""],
    [0x17B, "PICO_RESOLUTION_NOT_SUPPORTED_BY_VARIENT", ""],
    [0x17C, "PICO_THRESHOLD_OUT_OF_RANGE", ""],
    [0x17D, "PICO_INVALID_SIMPLE_TRIGGER_DIRECTION", ""],
    [0x17E, "PICO_AUX_NOT_SUPPORTED", ""],
    [0x17F, "PICO_NULL_DIRECTIONS", ""],
    [0x180, "PICO_NULL_CHANNEL_PROPERTIES", ""],
    [0x181, "PICO_TRIGGER_CHANNEL_NOT_ENABLED", ""],
    [0x182, "PICO_CONDITION_HAS_NO_TRIGGER_PROPERTY", ""],
    [0x183, "PICO_RATIO_MODE_TRIGGER_MASKING_INVALID", ""],
    [0x184, "PICO_TRIGGER_DATA_REQUIRES_MIN_BUFFER_SIZE_OF_40_SAMPLES", ""],
    [0x185, "PICO_NO_OF_CAPTURES_OUT_OF_RANGE", ""],
    [0x186, "PICO_RATIO_MODE_SEGMENT_HEADER_DOES_NOT_REQUIRE_BUFFERS", ""],
    [0x187, "PICO_FOR_SEGMENT_HEADER_USE_GETTRIGGERINFO", ""],
    [0x188, "PICO_READ_NOT_SET", ""],
    [0x189, "PICO_ADC_SETTING_MISMATCH", ""],
    [0x18A, "PICO_DATATYPE_INVALID", ""],
    [0x18B, "PICO_RATIO_MODE_DOES_NOT_SUPPORT_DATATYPE", ""],
    [0x18C, "PICO_CHANNEL_COMBINATION_NOT_VALID_IN_THIS_RESOLUTION", ""],
    [0x18D, "PICO_USE_8BIT_RESOLUTION", ""],
    [0x18E, "PICO_AGGREGATE_BUFFERS_SAME_POINTER", ""],
    [0x18F, "PICO_OVERLAPPED_READ_VALUES_OUT_OF_RANGE", ""],
    [0x190, "PICO_OVERLAPPED_READ_SEGMENTS_OUT_OF_RANGE", ""],
    [0x191, "PICO_CHANNELFLAGSCOMBINATIONS_ARRAY_SIZE_TOO_SMALL", ""],
    [0x192, "PICO_CAPTURES_EXCEEDS_NO_OF_SUPPORTED_SEGMENTS", ""],
    [0x193, "PICO_TIME_UNITS_OUT_OF_RANGE", ""],
    [0x194, "PICO_NO_SAMPLES_REQUESTED", ""],
    [0x195, "PICO_INVALID_ACTION", ""],
    [0x196, "PICO_NO_OF_SAMPLES_NEED_TO_BE_EQUAL_WHEN_ADDING_BUFFERS", ""],
    [0x197, "PICO_WAITING_FOR_DATA_BUFFERS", ""],
    [0x198, "PICO_STREAMING_ONLY_SUPPORTS_ONE_READ", ""],
    [0x199, "PICO_CLEAR_DATA_BUFFER_INVALID", ""],
    [0x19A, "PICO_INVALID_ACTION_FLAGS_COMBINATION", ""],
    [0x19B, "PICO_PICO_MOTH_MIN_AND_MAX_NULL_BUFFERS_CANNOT_BE_ADDED", ""],
    [0x19C,
     "PICO_CONFLICT_IN_SET_DATA_BUFFERS_CALL_REMOVE_DATA_BUFFER_TO_RESET", ""],
    [0x19D,
     "PICO_REMOVING_DATA_BUFFER_ENTRIES_NOT_ALLOWED_WHILE_DATA_PROCESSING",
     ""],
    [0x200, "PICO_CYUSB_REQUEST_FAILED", ""],
    [0x201, "PICO_STREAMING_DATA_REQUIRED", ""],
    [0x202, "PICO_INVALID_NUMBER_OF_SAMPLES", ""],
    [0x203, "PICO_INALID_DISTRIBUTION", ""],
    [0x204, "PICO_BUFFER_LENGTH_GREATER_THAN_INT32_T", ""],
    [0x209, "PICO_PLL_MUX_OUT_FAILED", ""],
    [0x20A, "PICO_ONE_PULSE_WIDTH_DIRECTION_ALLOWED", ""],
    [0x20B, "PICO_EXTERNAL_TRIGGER_NOT_SUPPORTED", ""],
    [0x20C, "PICO_NO_TRIGGER_CONDITIONS_SET", ""],
    [0x20D, "PICO_NO_OF_CHANNEL_TRIGGER_PROPERTIES_OUT_OF_RANGE", ""],
    [0x20E, "PICO_PROBE_COMPNENT_ERROR", ""],
    [0x210, "PICO_INVALID_TRIGGER_CHANNELS_FOR_ETS", ""],
    [0x211, "PICO_NOT_AVALIABLE_WHEN_STREAMING_IS_RUNNING", ""],
    [0x212, "PICO_INVALID_TRIGGER_WITHIN_PRE_TRIGGER_STATE", ""],
    [0x213, "PICO_ZERO_NUMBER_OF_CAPTURES_INVALID", ""],
    [0x300, "PICO_TRIGGER_DELAY_OUT_OF_RANGE", ""],
    [0x301, "PICO_INVALID_THRESHOLD_DIRECTION", ""],
    [0x302, "PICO_INVALID_THRESGOLD_MODE", ""],
    [0x6000, "PICO_HARDWARE_CAPTURE_TIMEOUT",
     "waiting for the device to capture timed out"],
    [0x6001, "PICO_HARDWARE_READY_TIMEOUT",
     "waiting for the device be ready for capture timed out"],
    [0x6002, "PICO_HARDWARE_CAPTURING_CALL_STOP",
     ("the driver is performing a capture requested by RunStreaming or "
      "RunBlock to interrupt this capture call Stop on the device first")],
    [0x2000001, "PICO_TRIGGER_TIME_NOT_REQUESTED",
     "Requesting the TriggerTimeOffset, the trigger time has not been set."],
    [0x1000000, "PICO_DEVICE_TIME_STAMP_RESET", ""],
    [0x10000000, "PICO_WATCHDOGTIMER", ""],
    [0x10000001, "PICO_IPP_NOT_FOUND", ""],
    [0x10000002, "PICO_IPP_NO_FUNCTION", ""],
    [0x10000003, "PICO_IPP_ERROR", ""],
    [0x10000004, "PICO_SHADOW_CAL_NOT_AVAILABLE", ""],
    [0x10000005, "PICO_SHADOW_CAL_DISABLED", ""],
    [0x10000006, "PICO_SHADOW_CAL_ERROR", ""],
    [0x10000007, "PICO_SHADOW_CAL_CORRUPT", ""],
]



"""
PS2000 Demo.

By: Colin O'Flynn, based on Mark Harfouche's software

This is a demo of how to use AWG with the Picoscope 2204 along with capture
It was tested with the PS2204A USB2.0 version

The AWG is connected to Channel A.
Nothing else is required.

NOTE: Must change line below to use with "A" and "B" series PS2000 models

See http://www.picotech.com/document/pdf/ps2000pg.en-10.pdf for PS2000 models:
PicoScope 2104
PicoScope 2105
PicoScope 2202
PicoScope 2203
PicoScope 2204
PicoScope 2205
PicoScope 2204A
PicoScope 2205A

See http://www.picotech.com/document/pdf/ps2000apg.en-6.pdf for PS2000A models:
PicoScope 2205 MSO
PicoScope 2206
PicoScope 2206A
PicoScope 2206B
PicoScope 2207
PicoScope 2207A
PicoScope 2208
PicoScope 2208A
"""
from __future__ import division
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import time
from picoscope import ps2000
# from picoscope import ps2000a
import pylab as plt
import numpy as np

if __name__ == "__main__":
    print(__doc__)

    print("Attempting to open Picoscope 2000...")

    ps = ps2000.PS2000()
    # Uncomment this line to use with the 2000a/2000b series
    # ps = ps2000a.PS2000a()

    print("Found the following picoscope:")
    print(ps.getAllUnitInfo())

    waveform_desired_duration = 50E-6
    obs_duration = 3 * waveform_desired_duration
    sampling_interval = obs_duration / 4096

    (actualSamplingInterval, nSamples, maxSamples) = \
        ps.setSamplingInterval(sampling_interval, obs_duration)
    print("Sampling interval = %f ns" % (actualSamplingInterval * 1E9))
    print("Taking  samples = %d" % nSamples)
    print("Maximum samples = %d" % maxSamples)

    # the setChannel command will chose the next largest amplitude
    channelRange = ps.setChannel('A', 'DC', 2.0, 0.0, enabled=True,
                                 BWLimited=False)
    print("Chosen channel range = %d" % channelRange)

    ps.setSimpleTrigger('A', 1.0, 'Falling', timeout_ms=100, enabled=True)

    ps.setSigGenBuiltInSimple(offsetVoltage=0, pkToPk=1.2, waveType="Sine",
                              frequency=50E3)

    ps.runBlock()
    ps.waitReady()
    print("Waiting for awg to settle.")
    time.sleep(2.0)
    ps.runBlock()
    ps.waitReady()
    print("Done waiting for trigger")
    dataA = ps.getDataV('A', nSamples, returnOverflow=False)

    dataTimeAxis = np.arange(nSamples) * actualSamplingInterval

    ps.stop()
    ps.close()

    # Uncomment following for call to .show() to not block
    # plt.ion()

    plt.figure()
    plt.hold(True)
    plt.plot(dataTimeAxis, dataA, label="Clock")
    plt.grid(True, which='major')
    plt.title("Picoscope 2000 waveforms")
    plt.ylabel("Voltage (V)")
    plt.xlabel("Time (ms)")
    plt.legend()
    plt.show()



"""
OpenUnitAsync Demo.

By: Mark Harfouche

Shows how to use the openUnitAsync functionality.
On my computer, it takes 2.8 seconds to open the picoscope.
Maybe you want to do something useful with it :D.
"""

from __future__ import division
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import time
from picoscope import ps6000

if __name__ == "__main__":
    print(__doc__)

    ps = ps6000.PS6000(connect=False)

    print("Attempting to open Picoscope 6000...")

    ps.openUnitAsync()

    t_start = time.time()
    while True:
        (progress, completed) = ps.openUnitProgress()
        print("T = %f, Progress = %d, Completed = %d" %
              (time.time() - t_start, progress, completed))
        if completed == 1:
            break
        time.sleep(0.01)

    print("Completed opening the scope in %f seconds." %
          (time.time() - t_start))

    ps.close()



# -*- coding: utf-8
#
# Colin O'Flynn, Copyright (C) 2013. All Rights Reserved. <coflynn@newae.com>
#
import time
import numpy as np
from picoscope import ps6000

import pylab as plt

import scipy
import scipy.fftpack


def fft(signal, freq):
    FFT = abs(scipy.fft(signal))
    FFTdb = 20*scipy.log10(FFT)
    freqs = scipy.fftpack.fftfreq(len(signal), 1/freq)

    FFTdb = FFTdb[2:len(freqs)/2]
    freqs = freqs[2:len(freqs)/2]

    return (freqs, FFTdb)


def examplePS6000():
    fig = plt.figure()  # noqa
    plt.ion()
    plt.show()

    print("Attempting to open...")
    ps = ps6000.PS6000()

    # Example of simple capture
    res = ps.setSamplingFrequency(250E6, 4096)
    sampleRate = res[0]  # noqa
    print("Sampling @ %f MHz, %d samples" % (res[0]/1E6, res[1]))
    ps.setChannel("A", "AC", 50E-3)

    blockdata = np.array(0)

    for i in range(0, 50):
        ps.runBlock()
        while not ps.isReady():
            time.sleep(0.01)

        print("Sampling Done")
        data = ps.getDataV("A", 4096)
        blockdata = np.append(blockdata, data)

        # Simple FFT
        # print "FFT In Progress"
        # [freqs, FFTdb] = fft(data, res[0])
        # plt.clf()
        # plt.plot(freqs, FFTdb)
        # plt.draw()

        start = (i - 5) * 4096
        if start < 0:
            start = 0
        # Spectrum Graph, keeps growing
        plt.clf()
        plt.specgram(blockdata[start:], NFFT=4096, Fs=res[0], noverlap=512)
        plt.xlabel('Measurement #')
        plt.ylabel('Frequency (Hz)')
        plt.draw()

    ps.close()


if __name__ == "__main__":
    examplePS6000()



__author__ = "Colin O'Flynn, Mark Harfouche"
__license__ = "FreeBSD"

__all__ = ["ps2000",
           "ps2000a",
           "ps3000",
           "ps3000a",
           "ps4000",
           "ps4000a",
           "ps5000a",
           "ps6000"]

from ._version import get_versions
__version__ = get_versions()['version']
del get_versions



"""
PS6000 Siggen Demo.

By: Mark Harfouche

This is a demo of how to use Siggen with the Picoscope 6000
It was tested with the PS6403B USB2.0 version

The system is very simple:

The SigGen is connected to Channel A. No other setup is required

Warning, the picoscope has a bug, that doesn't let you generate a
waveform correctly for a while
It means that you need to kick it in place

See http://www.picotech.com/support/topic12969.html

"""
from __future__ import division

import time
from picoscope import ps6000
import pylab as plt
import numpy as np

if __name__ == "__main__":
    print(__doc__)

    print("Attempting to open Picoscope 6000...")

    # see page 13 of the manual to understand how to work this beast
    ps = ps6000.PS6000()

    print(ps.getAllUnitInfo())

    waveform_desired_duration = 1E-3
    obs_duration = 10 * waveform_desired_duration
    sampling_interval = obs_duration / 4096

    (actualSamplingInterval, nSamples, maxSamples) = ps.setSamplingInterval(
        sampling_interval, obs_duration)
    print("Sampling interval = %f ns" % (actualSamplingInterval * 1E9))
    print("Taking  samples = %d" % nSamples)
    print("Maximum samples = %d" % maxSamples)

    ps.setChannel('A', 'DC', 5.0, 0.0, True, False)
    ps.setSimpleTrigger('A', 0.0, 'Rising', delay=0, timeout_ms=100,
                        enabled=True)

    ps.setSigGenBuiltInSimple(offsetVoltage=0, pkToPk=4, waveType="Square",
                              frequency=1 / waveform_desired_duration * 10,
                              shots=1, triggerType="Rising",
                              triggerSource="None")

    # take the desired waveform
    # This measures all the channels that have been enabled

    ps.runBlock()
    ps.waitReady()
    print("Done waiting for trigger")
    time.sleep(10)
    ps.runBlock()
    ps.waitReady()

    dataA = ps.getDataV('A', nSamples, returnOverflow=False)

    ps.stop()
    ps.close()

    dataTimeAxis = np.arange(nSamples) * actualSamplingInterval

    plt.ion()

    plt.figure()
    plt.hold(True)
    plt.plot(dataTimeAxis, dataA, label="Waveform")
    plt.grid(True, which='major')
    plt.title("Picoscope 6000 waveforms")
    plt.ylabel("Voltage (V)")
    plt.xlabel("Time (ms)")
    plt.legend()
    plt.draw()



"""
Shows a demo of use to use the enumerate devices feature.

By: Mark Harfouche

"""
from __future__ import division
from picoscope import ps6000
import time

if __name__ == "__main__":
    ps = ps6000.PS6000(connect=False)

    allSerialNumbers = ps.enumerateUnits()

    print("Found the following device serial numbers: ")
    for serial in allSerialNumbers:
        print(serial + "\n")

    # you can open devices by serial number
    serial = allSerialNumbers[0]
    ps = ps6000.PS6000(serial)

    # do stuff
    ps.flashLed(10)
    time.sleep(4.0)  # the above flash takes roughly 4 seconds

    ps.close()



# This is the instrument-specific file for the ps6000A series of instruments.
#
# pico-python is Copyright (c) 2013-2014 By:
# Colin O'Flynn <coflynn@newae.com>
# Mark Harfouche <mark.harfouche@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

"""
This is the low level driver file for a specific Picoscope.

By this, I mean if parameters want to get passed as strings, they should be
handled by PSBase
All functions here should take things as close to integers as possible, the
only exception here is for array parameters. Array parameters should be passed
in a pythonic way through numpy since the PSBase class should not be aware of
the specifics behind how the clib is called.

The functions should not have any default values as these should be handled
by PSBase.

"""

# 3.0 compatibility
# see http://docs.python.org/2/library/__future__.html
from __future__ import division
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import math

# to load the proper dll
import platform

# Do not import or use ill definied data types
# such as short int or long
# use the values specified in the h file
# float is always defined as 32 bits
# double is defined as 64 bits
from ctypes import byref, POINTER, create_string_buffer, c_float, c_int8, \
    c_double, c_int16, c_uint16, c_int32, c_uint32, c_int64, c_uint64, \
    c_void_p, CFUNCTYPE
from ctypes import c_int32 as c_enum

from picoscope.picobase import _PicoscopeBase


# Decorators for callback functions. PICO_STATUS is uint32_t.
def blockReady(function):
    """typedef void (*ps6000aBlockReady)
    (
     int16_t         handle,
     PICO_STATUS     status,
     void          * pParameter
    )
    """
    if function is None:
        return None
    callback = CFUNCTYPE(c_void_p, c_int16, c_uint32, c_void_p)
    return callback(function)


def dataReady(function):
    """typedef void (*ps6000aDataReady)
    (
     int16_t         handle,
     PICO_STATUS     status,
     uint64_t        noOfSamples,
     int16_t         overflow,
     void         * pParameter
    )
    """
    if function is None:
        return None
    callback = CFUNCTYPE(c_void_p, c_int16, c_uint32, c_uint64, c_int16,
                         c_void_p)
    return callback(function)


def updateFirmwareProgress(function):
    """typedef void (*PicoUpdateFirmwareProgress)
    (
     int16_t    handle,
     uint16_t   progress
    )
    """
    if function is None:
        return None
    callback = CFUNCTYPE(c_void_p, c_int16, c_uint16)
    return callback(function)


class PS6000a(_PicoscopeBase):
    """The following are low-level functions for the ps6000A.

    The 'TriggerAux' channel (trigger input at backside) works with
    setSimpleTrigger.

    Due to the new nature of the setDataBuffer method with actions, you have to
    clear all configured buffers before using a different readout method.
    """

    LIBNAME = "ps6000a"

    # Resolution in bit
    ADC_RESOLUTIONS = {"8": 0, "10": 10, "12": 1}

    NUM_CHANNELS = 4
    CHANNELS = {"A": 0, "B": 1, "C": 2, "D": 3,
                "External": 1000, "MaxChannels": 4, "TriggerAux": 1001}

    CHANNEL_COUPLINGS = {"DC50": 50, "DC": 1, "AC": 0}

    ACTIONS = {  # PICO_ACTION they can be combined with bitwise OR.
        'clear_all': 0x00000001,  # PICO_CLEAR_ALL
        'add': 0x00000002,  # PICO_ADD
        'clear_this': 0x00001000,  # PICO_CLEAR_THIS_DATA_BUFFER
        'clear_waveform': 0x00002000,  # PICO_CLEAR_WAVEFORM_DATA_BUFFERS
        'clear_waveform_read': 0x00004000,
        # PICO_CLEAR_WAVEFORM_READ_DATA_BUFFERS
    }

    DATA_TYPES = {  # PICO_DATA_TYPE
        'int8': 0,  # PICO_INT8_T
        'int16': 1,  # PICO_INT16_T
        'int32': 2,  # PICO_INT32_T
        'uint32': 3,  # PICO_UINT32_T
        'int64': 4,  # PICO_INT64_T
    }

    TIME_UNITS = [  # PICO_TIME_UNITS
        1e-15,  # PICO_FS
        1e-12,  # PICO_PS
        1e-9,  # PICO_NS
        1e-6,  # PICO_US
        1e-3,  # PICO_MS
        1,  # PICO_S
    ]

    # Only at 8 bit, use GetAdcLimits for other resolutions.
    MAX_VALUE = 32512
    MIN_VALUE = -32512

    # 10V and 20V are only allowed in high impedence modes.
    CHANNEL_RANGE = [{"rangeV": 20E-3,  "apivalue": 1, "rangeStr": "20 mV"},
                     {"rangeV": 50E-3,  "apivalue": 2, "rangeStr": "50 mV"},
                     {"rangeV": 100E-3, "apivalue": 3, "rangeStr": "100 mV"},
                     {"rangeV": 200E-3, "apivalue": 4, "rangeStr": "200 mV"},
                     {"rangeV": 500E-3, "apivalue": 5, "rangeStr": "500 mV"},
                     {"rangeV": 1.0,    "apivalue": 6, "rangeStr": "1 V"},
                     {"rangeV": 2.0,    "apivalue": 7, "rangeStr": "2 V"},
                     {"rangeV": 5.0,    "apivalue": 8, "rangeStr": "5 V"},
                     {"rangeV": 10.0,   "apivalue": 9, "rangeStr": "10 V"},
                     {"rangeV": 20.0,   "apivalue": 10, "rangeStr": "20 V"},
                     ]

    RATIO_MODE = {"aggregate": 1,  # max and min of every n data.
                  "decimate": 2,  # Take every n data.
                  "average": 4,  # Average of every n data.
                  "trigger": 0x40000000,  # 20 samples either side of the
                  # trigger. This cannot be combined with any other ratio mode
                  "raw": 0x80000000,  # No downsampling
                  "none": 0x80000000,  # for compatibility
                  }

    EXT_MAX_VALUE = 32767
    EXT_MIN_VALUE = -32767
    EXT_RANGE_VOLTS = 5

    # TODO verify AWG values
    WAVE_TYPES = {"Sine": 0, "Square": 1, "Triangle": 2,
                  "RampUp": 3, "RampDown": 4,
                  "Sinc": 5, "Gaussian": 6, "HalfSine": 7, "DCVoltage": 8,
                  "WhiteNoise": 9}

    SWEEP_TYPES = {"Up": 0, "Down": 1, "UpDown": 2, "DownUp": 3}

    SIGGEN_TRIGGER_TYPES = {"Rising": 0, "Falling": 1,
                            "GateHigh": 2, "GateLow": 3}
    SIGGEN_TRIGGER_SOURCES = {"None": 0, "ScopeTrig": 1, "AuxIn": 2,
                              "ExtIn": 3, "SoftTrig": 4, "TriggerRaw": 5}

    AWGPhaseAccumulatorSize = 32
    AWGBufferAddressWidth = 14
    AWGMaxSamples = 2 ** AWGBufferAddressWidth

    AWGDACInterval = 5E-9  # in seconds
    AWGDACFrequency = 1 / AWGDACInterval

    AWGMaxVal = 0x0FFF
    AWGMinVal = 0x0000

    AWG_INDEX_MODES = {"Single": 0, "Dual": 1, "Quad": 2}

    def __init__(self, serialNumber=None, connect=True, resolution="8"):
        """Load DLLs."""
        self.handle = None
        self.resolution = self.ADC_RESOLUTIONS.get(resolution)

        if platform.system() == 'Linux':
            from ctypes import cdll
            # ok I don't know what is wrong with my installer,
            # but I need to include .so.2
            self.lib = cdll.LoadLibrary("lib" + self.LIBNAME + ".so.2")
        elif platform.system() == 'Darwin':
            from picoscope.darwin_utils import LoadLibraryDarwin
            self.lib = LoadLibraryDarwin("lib" + self.LIBNAME + ".dylib")
        else:
            from ctypes import windll
            from ctypes.util import find_library
            self.lib = windll.LoadLibrary(
                find_library(str(self.LIBNAME + ".dll"))
            )

        super(PS6000a, self).__init__(serialNumber, connect)

    "General unit calls"

    # Open / close unit
    def _lowLevelOpenUnit(self, serialNumber):
        c_handle = c_int16()
        if serialNumber is not None:
            serialNumberStr = create_string_buffer(bytes(serialNumber,
                                                         encoding='utf-8'))
        else:
            serialNumberStr = None
        # Passing None is the same as passing NULL
        m = self.lib.ps6000aOpenUnit(byref(c_handle), serialNumberStr,
                                     self.resolution)
        self.checkResult(m)
        self.handle = c_handle.value
        mi, ma = self._lowLevelGetAdcLimits(self.resolution)
        self.MIN_VALUE, self.MAX_VALUE = mi, ma

    def _lowLevelOpenUnitAsync(self, serialNumber):
        c_status = c_int16()
        if serialNumber is not None:
            serialNumberStr = create_string_buffer(bytes(serialNumber,
                                                         encoding='utf-8'))
        else:
            serialNumberStr = None
        # Passing None is the same as passing NULL
        m = self.lib.ps6000aOpenUnitAsync(byref(c_status), serialNumberStr,
                                          self.resolution)
        self.checkResult(m)
        return c_status.value

    def _lowLevelOpenUnitProgress(self):
        complete = c_int16()
        progressPercent = c_int16()
        handle = c_int16()

        m = self.lib.ps6000aOpenUnitProgress(byref(handle),
                                             byref(progressPercent),
                                             byref(complete))
        self.checkResult(m)
        if complete.value != 0:
            self.handle = handle.value
            mi, ma = self._lowLevelGetAdcLimits(self.resolution)
            self.MIN_VALUE, self.MAX_VALUE = mi, ma
        return (progressPercent.value, complete.value)

    def _lowLevelCloseUnit(self):
        m = self.lib.ps6000aCloseUnit(c_int16(self.handle))
        self.checkResult(m)

    # Misc
    def _lowLevelEnumerateUnits(self):
        count = c_int16(0)
        serials = c_int8(0)
        serialLth = c_int16(0)
        m = self.lib.ps6000aEnumerateUnits(byref(count), byref(serials),
                                           byref(serialLth))
        self.checkResult(m)
        # a serial number is rouhgly 8 characters
        # an extra character for the comma
        # and an extra one for the space after the comma?
        # the extra two also work for the null termination
        serialLth = c_int16(count.value * (8 + 2))
        serials = create_string_buffer(serialLth.value + 1)

        m = self.lib.ps6000aEnumerateUnits(byref(count), serials,
                                           byref(serialLth))
        self.checkResult(m)

        serialList = str(serials.value.decode('utf-8')).split(',')

        serialList = [x.strip() for x in serialList]

        return serialList

    def _lowLevelFlashLed(self, times):
        # TODO verify as it does not work
        m = self.lib.ps6000aFlashLed(c_int16(self.handle), c_int16(times))
        self.checkResult(m)

    def _lowLevelGetUnitInfo(self, info):
        s = create_string_buffer(256)
        requiredSize = c_int16(0)

        m = self.lib.ps6000aGetUnitInfo(c_int16(self.handle), byref(s),
                                        c_int16(len(s)), byref(requiredSize),
                                        c_enum(info))
        self.checkResult(m)
        if requiredSize.value > len(s):
            s = create_string_buffer(requiredSize.value + 1)
            m = self.lib.ps6000aGetUnitInfo(c_int16(self.handle), byref(s),
                                            c_int16(len(s)),
                                            byref(requiredSize), c_enum(info))
            self.checkResult(m)

        # should this be ascii instead?
        # I think they are equivalent...
        return s.value.decode('utf-8')

    def _lowLevelPingUnit(self):
        """Check connection to picoscope and return the error."""
        return self.lib.ps6000aPingUnit(c_int16(self.handle))

    "Measurement"

    # Timebase
    def _lowLevelGetTimebase(self, timebase, noSamples, oversample,
                             segmentIndex):
        """
        Calculate the sampling interval and maximum number of samples.

        timebase
            Number of the selected timebase.
        noSamples
            Number of required samples.
        oversample
            Not used.
        segmentIndex
            Index of the segment to save samples in

        Return
        -------
        timeIntervalSeconds : float
            Time interval between two samples in s.
        maxSamples : int
            maximum number of samples available depending on channels
            and timebase chosen.
        """
        maxSamples = c_uint64()
        timeIntervalNanoSeconds = c_double()

        m = self.lib.ps6000aGetTimebase(c_int16(self.handle),
                                        c_uint32(timebase),
                                        c_uint64(noSamples),
                                        byref(timeIntervalNanoSeconds),
                                        byref(maxSamples),
                                        c_uint64(segmentIndex))
        self.checkResult(m)

        return (timeIntervalNanoSeconds.value / 1.0e9, maxSamples.value)

    @staticmethod
    def getTimeBaseNum(sampleTimeS):
        """Convert `sampleTimeS` in s to the integer timebase number."""
        maxSampleTime = (((2 ** 32 - 1) - 4) / 156250000)

        if sampleTimeS < 6.4E-9:
            timebase = math.floor(math.log(sampleTimeS * 5E9, 2))
            timebase = max(timebase, 0)
        else:
            # Otherwise in range 2^32-1
            if sampleTimeS > maxSampleTime:
                sampleTimeS = maxSampleTime

            timebase = math.floor((sampleTimeS * 156250000) + 4)

        return timebase

    @staticmethod
    def getTimestepFromTimebase(timebase):
        """Convert `timebase` index to sampletime in seconds."""
        if timebase < 5:
            dt = 2 ** timebase / 5E9
        else:
            dt = (timebase - 4) / 156250000.
        return dt

    # Device resolution
    def _lowLevelGetDeviceResolution(self):
        """Return the vertical resolution of the oscilloscope."""
        resolution = c_uint32()
        m = self.lib.ps6000aGetDeviceResolution(c_int16(self.handle),
                                                byref(resolution))
        self.checkResult(m)
        self.resolution = resolution.value
        for key, value in self.ADC_RESOLUTIONS.items():
            if value == self.resolution:
                return key
        raise TypeError("Unknown resolution {}.".format(resolution))

    def _lowLevelSetDeviceResolution(self, resolution):
        """
        Set the sampling resolution of the device.

        At 10-bit and higher resolutions, the maximum capture buffer length is
        half that of 8-bit mode. When using 12-bit resolution only 2 channels
        can be enabled to capture data.
        """
        if type(resolution) is str:
            resolution = self.ADC_RESOLUTIONS[resolution]
        m = self.lib.ps6000aSetDeviceResolution(c_int16(self.handle),
                                                resolution)
        self.checkResult(m)
        self.resolution = resolution
        self.MIN_VALUE, self.MAX_VALUE = self._lowLevelGetAdcLimits(resolution)

    def _lowLevelGetAdcLimits(self, resolution):
        """
        This function gets the maximum and minimum sample values that the ADC
        can produce at a given resolution.
        """
        if type(resolution) is str:
            resolution = self.ADC_RESOLUTIONS[resolution]
        minimum = c_int16()
        maximum = c_int16()
        m = self.lib.ps6000aGetAdcLimits(c_int16(self.handle),
                                         resolution,
                                         byref(minimum),
                                         byref(maximum))
        self.checkResult(m)
        return minimum.value, maximum.value

    # Channel
    def _lowLevelSetChannel(self, chNum, enabled, coupling, VRange, VOffset,
                            BWLimited):
        if enabled:
            m = self.lib.ps6000aSetChannelOn(c_int16(self.handle),
                                             c_enum(chNum), c_enum(coupling),
                                             c_enum(VRange), c_double(VOffset),
                                             c_enum(BWLimited))
        else:
            m = self.lib.ps6000aSetChannelOff(c_int16(self.handle),
                                              c_enum(chNum))
        self.checkResult(m)

    # Trigger
    def _lowLevelSetSimpleTrigger(self, enabled, trigsrc, threshold_adc,
                                  direction, delay, timeout_ms):
        m = self.lib.ps6000aSetSimpleTrigger(c_int16(self.handle),
                                             c_int16(enabled),
                                             c_enum(trigsrc),
                                             c_int16(threshold_adc),
                                             c_enum(direction),
                                             c_uint64(delay),
                                             c_uint32(timeout_ms * 1000))
        self.checkResult(m)

    # Start / stop measurement
    def _lowLevelStop(self):
        m = self.lib.ps6000aStop(c_int16(self.handle))
        self.checkResult(m)

    def _lowLevelRunBlock(self, numPreTrigSamples, numPostTrigSamples,
                          timebase, oversample, segmentIndex, callback,
                          pParameter):
        # Hold a reference to the callback so that the Python
        # function pointer doesn't get free'd.
        self._c_runBlock_callback = blockReady(callback)
        timeIndisposedMs = c_int32()
        m = self.lib.ps6000aRunBlock(c_int16(self.handle),
                                     c_uint64(numPreTrigSamples),
                                     c_uint64(numPostTrigSamples),
                                     c_uint32(timebase),
                                     byref(timeIndisposedMs),
                                     c_uint64(segmentIndex),
                                     self._c_runBlock_callback,
                                     c_void_p())
        self.checkResult(m)
        return timeIndisposedMs.value

    def _lowLevelIsReady(self):
        ready = c_int16()
        m = self.lib.ps6000aIsReady(c_int16(self.handle), byref(ready))
        self.checkResult(m)
        if ready.value:
            return True
        else:
            return False

    # Setup data acquisition.
    def _lowLevelMemorySegments(self, nSegments):
        nMaxSamples = c_uint64()
        m = self.lib.ps6000aMemorySegments(c_int16(self.handle),
                                           c_uint64(nSegments),
                                           byref(nMaxSamples))
        self.checkResult(m)
        return nMaxSamples.value

    def _lowLevelSetDataBuffer(self, channel, data, downSampleMode,
                               segmentIndex):
        """Set the data buffer.

        Be sure to call _lowLevelClearDataBuffer
        when you are done with the data array
        or else subsequent calls to GetValue will still use the same array.

        action
            the method to use when creating the buffer. The buffers are added
            to a unique list for the channel, data type and segment. Therefore
            you must use PICO_CLEAR_ALL to remove all buffers already written.
            PICO_ACTION values can be ORed together to allow clearing and
            adding in one call.
        """
        if downSampleMode == 0:
            downSampleMode = self.RATIO_MODE['raw']
        dataPtr = data.ctypes.data_as(POINTER(c_int16))
        numSamples = len(data)

        m = self.lib.ps6000aSetDataBuffer(c_int16(self.handle),
                                          c_enum(channel),
                                          dataPtr,
                                          c_int32(numSamples),
                                          self.DATA_TYPES['int16'],
                                          c_uint64(segmentIndex),
                                          c_enum(downSampleMode),
                                          self.ACTIONS['add'])
        self.checkResult(m)

    def _lowLevelClearDataBuffer(self, channel, segmentIndex,
                                 downSampleMode=0):
        """Clear the buffer for the chosen channel, segment, downSampleMode."""
        if downSampleMode == 0:
            downSampleMode = self.RATIO_MODE['raw']
        m = self.lib.ps6000aSetDataBuffer(c_int16(self.handle),
                                          c_enum(channel),
                                          c_void_p(),
                                          c_int32(0),
                                          self.DATA_TYPES['int16'],
                                          c_uint64(segmentIndex),
                                          c_enum(downSampleMode),
                                          self.ACTIONS['clear_this'])
        self.checkResult(m)

    def _lowLevelClearDataBufferAll(self, channel=1, segmentIndex=0):
        """Clear all the stored buffers for all channels."""
        m = self.lib.ps6000aSetDataBuffer(c_int16(self.handle),
                                          c_enum(channel),
                                          c_void_p(),
                                          c_int32(0),
                                          self.DATA_TYPES['int16'],
                                          c_uint64(segmentIndex),
                                          c_enum(0),
                                          self.ACTIONS['clear_all'])
        self.checkResult(m)

    def _lowLevelSetDataBufferBulk(self, channel, data, segmentIndex,
                                   downSampleMode):
        """Just calls setDataBuffer with argument order changed.

        For compatibility with current picobase.py.
        """
        self._lowLevelSetDataBuffer(channel, data, downSampleMode,
                                    segmentIndex)

    # Acquire data.
    def _lowLevelGetValues(self, numSamples, startIndex, downSampleRatio,
                           downSampleMode, segmentIndex):
        if downSampleMode == 0:
            downSampleMode = self.RATIO_MODE['raw']
        numSamplesReturned = c_uint64()
        numSamplesReturned.value = numSamples
        overflow = c_int16()
        m = self.lib.ps6000aGetValues(c_int16(self.handle),
                                      c_uint64(startIndex),
                                      byref(numSamplesReturned),
                                      c_uint64(downSampleRatio),
                                      c_enum(downSampleMode),
                                      c_uint64(segmentIndex),
                                      byref(overflow))
        self.checkResult(m)
        return (numSamplesReturned.value, overflow.value)

    def _lowLevelGetValuesBulk(self, numSamples, fromSegmentIndex,
                               toSegmentIndex, downSampleRatio, downSampleMode,
                               overflow):
        if downSampleMode == 0:
            downSampleMode = self.RATIO_MODE['raw']
        overflowPoint = overflow.ctypes.data_as(POINTER(c_int16))
        m = self.lib.ps6000aGetValuesBulk(
            c_int16(self.handle),
            c_uint64(0),  # startIndex
            byref(c_int64(numSamples)),
            c_int64(fromSegmentIndex),
            c_int64(toSegmentIndex),
            c_int64(downSampleRatio),
            c_enum(downSampleMode),
            overflowPoint
        )
        self.checkResult(m)

    def _lowLevelGetValuesAsync(self, numSamples, startIndex, downSampleRatio,
                                downSampleMode, segmentIndex, callback, pPar):
        if downSampleMode == 0:
            downSampleMode = self.RATIO_MODE['raw']
        self._c_getValues_callback = dataReady(callback)
        m = self.lib.ps6000aGetValuesAsync(c_int16(self.handle),
                                           c_uint64(startIndex),
                                           c_uint64(numSamples),
                                           c_uint64(downSampleRatio),
                                           c_enum(downSampleMode),
                                           c_uint64(segmentIndex),
                                           self._c_getValues_callback,
                                           c_void_p())
        self.checkResult(m)

    # Misc
    def _lowLevelGetTriggerTimeOffset(self, segmentIndex):
        time = c_int64()
        timeUnits = c_enum()

        m = self.lib.ps6000aGetTriggerTimeOffset(c_int16(self.handle),
                                                 byref(time),
                                                 byref(timeUnits),
                                                 c_uint64(segmentIndex))
        self.checkResult(m)

        try:
            return time.value * self.TIME_UNITS[timeUnits.value]
        except KeyError:
            raise TypeError("Unknown timeUnits %d" % timeUnits.value)

    ###########################
    # TODO test functions below
    ###########################

    "Updates"

    def _lowLevelCheckForUpdate(self):
        """
        Check whether a firmware update for the device is available.

        Returns
        -------
        firmwareInfos : None
            not implemented: A struct with information.
        number : int
            Number of elements in the structure.
        required : bool
            Whether an update is required or not.
        """
        # TODO raises PICO_STRING_BUFFER_TOO_SMALL
        firmware_info = create_string_buffer(25600000)
        number = c_int16()
        required = c_uint16()
        m = self.lib.ps6000aCheckForUpdate(c_int16(self.handle),
                                           byref(firmware_info),
                                           byref(number), byref(required))
        self.checkResult(m)
        return firmware_info, number, required

    def _lowLevelStartFirmwareUpdate(self, function):
        # Hold a reference to the callback so that the Python
        # function pointer doesn't get free'd.
        self._c_updateFirmware_callback = updateFirmwareProgress(function)
        m = self.lib.ps6000aStartFirmwareUpdate(
            c_int16(self.handle),
            self._c_updateFirmware_callback)
        self.checkResult(m)

    "Misc"

    ##################################
    # TODO verify functions below here in the manual
    # TODO ensure all relevant functions are in here
    ##################################

    "Measurement"

    # Complicated triggering
    # need to understand structs for some of this to work
    def _lowLevelGetValuesTriggerTimeOffsetBulk():
        raise NotImplementedError()

    def _lowLevelSetTriggerChannelConditions():
        raise NotImplementedError()

    def _lowLevelSetTriggerChannelDirections():
        raise NotImplementedError()

    def _lowLevelSetTriggerChannelProperties():
        raise NotImplementedError()

    def _lowLevelSetTriggerDelay():
        raise NotImplementedError()

    def _lowLevelSetTriggerDigitalPortProperties(self):
        raise NotImplementedError()

    # Optional input triggering: PulseWidthQualifier
    def _lowLevelSetPulseWidthQualifierConditions(self):
        raise NotImplementedError()

    def _lowLevelSetPulseWidthQualifierDirections(self):
        raise NotImplementedError()

    def _lowLevelSetPulseWidthQualifierProperties(self):
        raise NotImplementedError()

    def _lowLevelTriggerWithinPreTriggerSamples(self):
        raise NotImplementedError()

    # Data acquisition
    def _lowLevelSetDataBuffers(self, channel, bufferMax, bufferMin,
                                downSampleMode):
        if downSampleMode == 0:
            downSampleMode = self.RATIO_MODE['raw']
        raise NotImplementedError()
        bufferMaxPtr = bufferMax.ctypes.data_as(POINTER(c_int16))
        bufferMinPtr = bufferMin.ctypes.data_as(POINTER(c_int16))
        bufferLth = len(bufferMax)

        m = self.lib.ps6000aSetDataBuffers(c_int16(self.handle),
                                           c_enum(channel),
                                           bufferMaxPtr, bufferMinPtr,
                                           c_uint32(bufferLth),
                                           c_enum(downSampleMode))
        self.checkResult(m)

    def _lowLevelClearDataBuffers(self, channel):
        raise NotImplementedError()
        m = self.lib.ps6000aSetDataBuffers(
            c_int16(self.handle), c_enum(channel),
            c_void_p(), c_void_p(), c_uint32(0), c_enum(0))
        self.checkResult(m)

    # Bulk values.
    # These would be nice, but the user would have to provide us
    # with an array.
    # we would have to make sure that it is contiguous amonts other things
    def _lowLevelSetNoOfCaptures(self, nCaptures):
        m = self.lib.ps6000aSetNoOfCaptures(c_int16(self.handle),
                                            c_uint32(nCaptures))
        self.checkResult(m)

    # Async functions
    def _lowLevelGetValuesBulkAsync():
        raise NotImplementedError()

    # overlapped functions
    def _lowLevelGetValuesOverlapped():
        raise NotImplementedError()

    # Streaming related functions
    def _lowLevelGetStreamingLatestValues():
        raise NotImplementedError()

    def _lowLevelNoOfStreamingValues(self):
        raise NotImplementedError()
        noOfValues = c_uint32()

        m = self.lib.ps6000aNoOfStreamingValues(c_int16(self.handle),
                                                byref(noOfValues))
        self.checkResult(m)

        return noOfValues.value

    def _lowLevelRunStreaming():
        raise NotImplementedError()

    "alphabetically"

    def _lowLevelChannelCombinationsStateless(self, resolution, timebase):
        """
        Return a list of the possible channel combinations given a proposed
        configuration (`resolution` and `timebase` number) of the oscilloscope.
        It does not change the configuration of the oscilloscope.

        Bit values of the different flags in a channel combination:
            PICO_CHANNEL_A_FLAGS = 1,
            PICO_CHANNEL_B_FLAGS = 2,
            PICO_CHANNEL_C_FLAGS = 4,
            PICO_CHANNEL_D_FLAGS = 8,
            PICO_CHANNEL_E_FLAGS = 16,
            PICO_CHANNEL_F_FLAGS = 32,
            PICO_CHANNEL_G_FLAGS = 64,
            PICO_CHANNEL_H_FLAGS = 128,
            PICO_PORT0_FLAGS = 65536,
            PICO_PORT1_FLAGS = 131072,
            PICO_PORT2_FLAGS = 262144,
            PICO_PORT3_FLAGS = 524288,
        """
        # TODO raises PICO_CHANNELFLAGSCOMBINATIONS_ARRAY_SIZE_TOO_SMALL
        ChannelCombinations = create_string_buffer(b"", 100000)
        nChannelCombinations = c_uint32()
        if isinstance(resolution, str):
            resolution = self.ADC_RESOLUTIONS[resolution]
        m = self.lib.ps6000aChannelCombinationsStateless(c_int16(self.handle),
                                                         ChannelCombinations,
                                                         nChannelCombinations,
                                                         c_uint32(resolution),
                                                         c_uint32(timebase),
                                                         )
        self.checkResult(m)
        return ChannelCombinations

    def _lowLevelGetAnalogueOffsetLimits(self, range, coupling):
        raise NotImplementedError()
        # TODO, populate properties with this function
        maximumVoltage = c_float()
        minimumVoltage = c_float()

        m = self.lib.ps6000aGetAnalogueOffsetLimits(
            c_int16(self.handle), c_enum(range), c_enum(coupling),
            byref(maximumVoltage), byref(minimumVoltage))
        self.checkResult(m)

        return (maximumVoltage.value, minimumVoltage.value)

    def _lowLevelGetMaximumAvailableMemory(self):
        raise NotImplementedError()

    def _lowLevelMinimumTimebaseStateless(self):
        raise NotImplementedError()

    def _lowLevelGetNoOfCaptures(self):
        raise NotImplementedError()
        nCaptures = c_uint32()

        m = self.lib.ps6000aGetNoOfCaptures(c_int16(self.handle),
                                            byref(nCaptures))
        self.checkResult(m)

        return nCaptures.value

    def _lowLevelGetNoOfProcessedCaptures(self):
        raise NotImplementedError()

    def _lowLevelGetTriggerInfo(self):
        raise NotImplementedError()

    def _lowLevelMemorySegmentsBySamples(self):
        raise NotImplementedError()

    def _lowLevelNearestSampleIntervalStateless(self):
        raise NotImplementedError()

    def _lowLevelQueryMaxSegmentsBySamples(self):
        raise NotImplementedError()

    def _lowLevelQueryOutputEdgeDetect(self):
        raise NotImplementedError()

    def _lowLevelSetDigitalPortOff(self):
        raise NotImplementedError()

    def _lowLevelSetDigitalPortOn(self):
        raise NotImplementedError()

    def _lowLevelSetOutputEdgeDetect(self):
        raise NotImplementedError()

    # Signal Generator
    # TODO add signal generator, at least in a simple version.
    def _lowLevelSigGenApply(self):
        raise NotImplementedError()

    def _lowLevelSigGenClockManual(self):
        raise NotImplementedError()

    def _lowLevelSigGenFilter(self):
        raise NotImplementedError()

    def _lowLevelSigGenFrequency(self):
        raise NotImplementedError()

    def _lowLevelSigGenFrequencyLimits(self):
        raise NotImplementedError()

    def _lowLevelSigGenFrequencySweep(self):
        raise NotImplementedError()

    def _lowLevelSigGenLimits(self):
        raise NotImplementedError()

    def _lowLevelSigGenPause(self):
        raise NotImplementedError()

    def _lowLevelSigGenPhase(self):
        raise NotImplementedError()

    def _lowLevelSigGenPhaseSweep(self):
        raise NotImplementedError()

    def _lowLevelSigGenRange(self):
        raise NotImplementedError()

    def _lowLevelSigGenRestart(self):
        raise NotImplementedError()

    def _lowLevelSigGenSoftwareTriggerControl(self):
        raise NotImplementedError()

    def _lowLevelSigGenTrigger(self):
        raise NotImplementedError()

    def _lowLevelSigGenWaveform(self):
        raise NotImplementedError()

    def _lowLevelSigGenWaveformDutyCycle(self):
        raise NotImplementedError()



# This is the instrument-specific file for the PS5000 series of instruments.
#
# pico-python is Copyright (c) 2013-2014 By:
# Colin O'Flynn <coflynn@newae.com>
# Mark Harfouche <mark.harfouche@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

"""
This is the low level driver file for a specific Picoscope.

By this, I mean if parameters want to get passed as strings, they should be
handled by PSBase
All functions here should take things as close to integers as possible, the
only exception here is for array parameters. Array parameters should be passed
in a pythonic way through numpy since the PSBase class should not be aware of
the specifics behind how the clib is called.

The functions should not have any default values as these should be handled
by PSBase.
"""

from __future__ import division
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import math

# to load the proper dll
import platform

# Do not import or use ill definied data types
# such as short int or long
# use the values specified in the h file
# float is always defined as 32 bits
# double is defined as 64 bits
from ctypes import byref, POINTER, create_string_buffer, c_float, \
    c_int16, c_int32, c_uint32, c_void_p, c_int64, CFUNCTYPE
from ctypes import c_int32 as c_enum

from picoscope.picobase import _PicoscopeBase


# Decorators for callback functions. PICO_STATUS is uint32_t.
def blockReady(function):
    """typedef void (*ps5000aBlockReady)
    (
     int16_t         handle,
     PICO_STATUS     status,
     void          * pParameter
    )
    """
    if function is None:
        return None

    callback = CFUNCTYPE(c_void_p, c_int16, c_uint32, c_void_p)
    return callback(function)


class PS5000a(_PicoscopeBase):
    """The following are low-level functions for the PS5000."""

    LIBNAME = "ps5000a"

    NUM_CHANNELS = 4
    CHANNELS = {"A": 0, "B": 1, "C": 2, "D": 3,
                "External": 4, "MaxChannels": 4, "TriggerAux": 5}

    ADC_RESOLUTIONS = {"8": 0, "12": 1, "14": 2, "15": 3, "16": 4}

    CHANNEL_RANGE = [{"rangeV": 10E-3, "apivalue": 0, "rangeStr": "10 mV"},
                     {"rangeV": 20E-3, "apivalue": 1, "rangeStr": "20 mV"},
                     {"rangeV": 50E-3, "apivalue": 2, "rangeStr": "50 mV"},
                     {"rangeV": 100E-3, "apivalue": 3, "rangeStr": "100 mV"},
                     {"rangeV": 200E-3, "apivalue": 4, "rangeStr": "200 mV"},
                     {"rangeV": 500E-3, "apivalue": 5, "rangeStr": "500 mV"},
                     {"rangeV": 1.0, "apivalue": 6, "rangeStr": "1 V"},
                     {"rangeV": 2.0, "apivalue": 7, "rangeStr": "2 V"},
                     {"rangeV": 5.0, "apivalue": 8, "rangeStr": "5 V"},
                     {"rangeV": 10.0, "apivalue": 9, "rangeStr": "10 V"},
                     {"rangeV": 20.0, "apivalue": 10, "rangeStr": "20 V"},
                     {"rangeV": 50.0, "apivalue": 11, "rangeStr": "50 V"},
                     ]

    CHANNEL_COUPLINGS = {"DC": 1, "AC": 0}

    # has_sig_gen = True
    WAVE_TYPES = {"Sine": 0, "Square": 1, "Triangle": 2,
                  "RampUp": 3, "RampDown": 4,
                  "Sinc": 5, "Gaussian": 6, "HalfSine": 7, "DCVoltage": 8,
                  "WhiteNoise": 9}

    SWEEP_TYPES = {"Up": 0, "Down": 1, "UpDown": 2, "DownUp": 3}

    SIGGEN_TRIGGER_TYPES = {"Rising": 0, "Falling": 1,
                            "GateHigh": 2, "GateLow": 3}
    SIGGEN_TRIGGER_SOURCES = {"None": 0, "ScopeTrig": 1, "AuxIn": 2,
                              "ExtIn": 3, "SoftTrig": 4, "TriggerRaw": 5}

    # This is actually different depending on the AB/CD models
    # I wonder how we could detect the difference between the oscilloscopes
    # I believe we can obtain this information from the setInfo function
    # by readign the hardware version
    # for the PS6403B version, the hardware version is "1 1",
    # an other possibility is that the PS6403B shows up as 6403 when using
    # VARIANT_INFO and others show up as PS6403X where X = A,C or D

    AWGPhaseAccumulatorSize = 32

    AWGDACInterval = 5E-9  # in seconds
    AWGDACFrequency = 1 / AWGDACInterval

    AWG_INDEX_MODES = {"Single": 0, "Dual": 1, "Quad": 2}

    MAX_VALUE_8BIT = 32512
    MIN_VALUE_8BIT = -32512
    MAX_VALUE_OTHER = 32767
    MIN_VALUE_OTHER = -32767

    EXT_RANGE_VOLTS = 5

    def __init__(self, serialNumber=None, connect=True):
        """Load DLL etc."""
        if platform.system() == 'Linux':
            from ctypes import cdll
            self.lib = cdll.LoadLibrary("lib" + self.LIBNAME + ".so")
        elif platform.system() == 'Darwin':
            from picoscope.darwin_utils import LoadLibraryDarwin
            self.lib = LoadLibraryDarwin("lib" + self.LIBNAME + ".dylib")
        else:
            from ctypes import windll
            from ctypes.util import find_library
            self.lib = windll.LoadLibrary(
                find_library(str(self.LIBNAME + ".dll"))
            )

        self.resolution = self.ADC_RESOLUTIONS["8"]

        super(PS5000a, self).__init__(serialNumber, connect)

    def _lowLevelOpenUnit(self, serialNumber):
        c_handle = c_int16()
        if serialNumber is not None:
            serialNumberStr = create_string_buffer(bytes(serialNumber,
                                                         encoding='utf-8'))
        else:
            serialNumberStr = None
        # Passing None is the same as passing NULL
        m = self.lib.ps5000aOpenUnit(byref(c_handle), serialNumberStr,
                                     self.resolution)
        self.handle = c_handle.value

        # This will check if the power supply is not connected
        # and change the power supply accordingly
        # Personally (me = Mark), I don't like this
        # since the user should address this immediately, and we
        # shouldn't let this go as a soft error
        # but I think this should do for now
        if m == 0x11A:
            self.changePowerSource(m)
        else:
            self.checkResult(m)

        # B models have different AWG buffer sizes
        # 5242B, 5442B: 2**14
        # 5243B, 5443B: 2**15
        # 5444B, 5244B: 3 * 2**14
        # Model 5444B identifies itself properly in VariantInfo, I will assume
        # the others do as well.

        self.model = self.getUnitInfo('VariantInfo')
        # print("Checking variant, found: " + str(self.model))
        if self.model in ('5244B', '5444B'):
            self.AWGBufferAddressWidth = math.log(3 * 2**14, 2)
            self.AWGMaxVal = 32767
            self.AWGMinVal = -32768
            self.AWGMaxSamples = 49152
        elif self.model in ('5243B', '5443B', '5243D', '5443D'):
            self.AWGBufferAddressWidth = 15
            self.AWGMaxVal = 32767
            self.AWGMinVal = -32768
            self.AWGMaxSamples = 2**self.AWGBufferAddressWidth
        else:
            # This is what the previous PS5000a used for all scopes.
            # I am leaving it the same, although I think the AWGMaxVal and
            # AWGMinVal issue was fixed and should be -32768 to 32767 for all
            # 5000 models
            self.AWGBufferAddressWidth = 14
            # Note this is NOT what is written in the Programming guide as of
            # version # 10_5_0_28
            # This issue was acknowledged in this thread
            # http://www.picotech.com/support/topic13217.html
            self.AWGMaxVal = 0x0FFF
            self.AWGMinVal = 0x0000
            self.AWGMaxSamples = 2**self.AWGBufferAddressWidth

    def _lowLevelCloseUnit(self):
        m = self.lib.ps5000aCloseUnit(c_int16(self.handle))
        self.checkResult(m)

    def _lowLevelSetChannel(self, chNum, enabled, coupling, VRange, VOffset,
                            bandwidth):
        m = self.lib.ps5000aSetChannel(c_int16(self.handle), c_enum(chNum),
                                       c_int16(enabled), c_enum(coupling),
                                       c_enum(VRange), c_float(VOffset))
        self.checkResult(m)

        # The error this might through are
        #    INVALID_HANDLE
        #    INVALID_CHANNEL
        #    INVALID_BANDWIDTH
        # Invalid bandwidth is the only case that could go wrong.
        # The others would be thrown above (assuming no race condition:
        # i.e. unplugging the scope in between this call.
        # I decided to keep the logic below to avoid a possible error
        # picobase/SetChannel should be changed to the following
        # Set the channel
        # save the new channel settings
        # check if ps5000a
        # change the bandwidth separately
        # changing the bandwidth would be it's own function (implemented below)
        if bandwidth:
            m = self.lib.ps5000aSetBandwidthFilter(c_int16(self.handle),
                                                   c_enum(chNum), c_enum(1))
        else:
            m = self.lib.ps5000aSetBandwidthFilter(c_int16(self.handle),
                                                   c_enum(chNum), c_enum(0))
        self.checkResult(m)

    def _lowLevelSetBandwidthFilter(self, channel, bandwidth):
        m = self.lib.ps5000aSetBandwidthFilter(c_int16(self.handle),
                                               c_enum(channel),
                                               c_enum(bandwidth))
        self.checkResult(m)

    def _lowLevelStop(self):
        m = self.lib.ps5000aStop(c_int16(self.handle))
        self.checkResult(m)

    def _lowLevelGetUnitInfo(self, info):
        s = create_string_buffer(256)
        requiredSize = c_int16(0)

        m = self.lib.ps5000aGetUnitInfo(c_int16(self.handle), byref(s),
                                        c_int16(len(s)), byref(requiredSize),
                                        c_enum(info))
        self.checkResult(m)
        if requiredSize.value > len(s):
            s = create_string_buffer(requiredSize.value + 1)
            m = self.lib.ps5000aGetUnitInfo(c_int16(self.handle), byref(s),
                                            c_int16(len(s)),
                                            byref(requiredSize), c_enum(info))
            self.checkResult(m)

        # should this bee ascii instead?
        # I think they are equivalent...
        return s.value.decode('utf-8')

    def _lowLevelFlashLed(self, times):
        m = self.lib.ps5000aFlashLed(c_int16(self.handle), c_int16(times))
        self.checkResult(m)

    def _lowLevelSetSimpleTrigger(self, enabled, trigsrc, threshold_adc,
                                  direction, delay, timeout_ms):
        m = self.lib.ps5000aSetSimpleTrigger(
            c_int16(self.handle), c_int16(enabled),
            c_enum(trigsrc), c_int16(threshold_adc),
            c_enum(direction), c_uint32(delay), c_int16(timeout_ms))
        self.checkResult(m)

    def _lowLevelRunBlock(self, numPreTrigSamples, numPostTrigSamples,
                          timebase, oversample, segmentIndex, callback,
                          pParameter):
        # Hold a reference to the callback so that the Python
        # function pointer doesn't get free'd.
        self._c_runBlock_callback = blockReady(callback)
        timeIndisposedMs = c_int32()
        m = self.lib.ps5000aRunBlock(
            c_int16(self.handle), c_uint32(numPreTrigSamples),
            c_uint32(numPostTrigSamples), c_uint32(timebase),
            byref(timeIndisposedMs), c_uint32(segmentIndex),
            self._c_runBlock_callback, c_void_p())
        self.checkResult(m)
        return timeIndisposedMs.value

    def _lowLevelIsReady(self):
        ready = c_int16()
        m = self.lib.ps5000aIsReady(c_int16(self.handle), byref(ready))
        self.checkResult(m)
        if ready.value:
            return True
        else:
            return False

    def _lowLevelPingUnit(self):
        m = self.lib.ps5000aPingUnit(c_int16(self.handle))
        return m

    def _lowLevelGetTimebase(self, tb, noSamples, oversample, segmentIndex):
        """Return (timeIntervalSeconds, maxSamples)."""
        maxSamples = c_int32()
        sampleRate = c_float()

        m = self.lib.ps5000aGetTimebase2(c_int16(self.handle), c_uint32(tb),
                                         c_uint32(noSamples),
                                         byref(sampleRate),
                                         byref(maxSamples),
                                         c_uint32(segmentIndex))
        self.checkResult(m)

        return (sampleRate.value / 1.0E9, maxSamples.value)

    def getTimeBaseNum(self, sampleTimeS):
        """Convert sample time in S to something to pass to API Call."""
        if self.resolution == self.ADC_RESOLUTIONS["8"]:
            maxSampleTime = (((2 ** 32 - 1) - 2) / 125000000)
            if sampleTimeS < 8.0E-9:
                st = math.floor(math.log(sampleTimeS * 1E9, 2))
                st = max(st, 0)
            else:
                if sampleTimeS > maxSampleTime:
                    sampleTimeS = maxSampleTime
                st = math.floor((sampleTimeS * 125000000) + 2)

        elif self.resolution == self.ADC_RESOLUTIONS["12"]:
            maxSampleTime = (((2 ** 32 - 1) - 3) / 62500000)
            if sampleTimeS < 16.0E-9:
                st = math.floor(math.log(sampleTimeS * 5E8, 2)) + 1
                st = max(st, 1)
            else:
                if sampleTimeS > maxSampleTime:
                    sampleTimeS = maxSampleTime
                st = math.floor((sampleTimeS * 62500000) + 3)

        elif (self.resolution == self.ADC_RESOLUTIONS["14"]) or (
                self.resolution == self.ADC_RESOLUTIONS["15"]):
            maxSampleTime = (((2 ** 32 - 1) - 2) / 125000000)
            if sampleTimeS > maxSampleTime:
                sampleTimeS = maxSampleTime
            st = math.floor((sampleTimeS * 125000000) + 2)
            st = max(st, 3)

        elif self.resolution == self.ADC_RESOLUTIONS["16"]:
            maxSampleTime = (((2 ** 32 - 1) - 3) / 62500000)
            if sampleTimeS > maxSampleTime:
                sampleTimeS = maxSampleTime
            st = math.floor((sampleTimeS * 62500000) + 3)
            st = max(st, 3)

        else:
            raise ValueError("Invalid Resolution for Device?")

        # is this cast needed?
        st = int(st)
        return st

    def getTimestepFromTimebase(self, timebase):
        """Return Timestep from timebase."""
        if self.resolution == self.ADC_RESOLUTIONS["8"]:
            if timebase < 3:
                dt = 2. ** timebase / 1.0E9
            else:
                dt = (timebase - 2.0) / 125000000.
        elif self.resolution == self.ADC_RESOLUTIONS["12"]:
            if timebase < 4:
                dt = 2. ** (timebase - 1) / 5.0E8
            else:
                dt = (timebase - 3.0) / 62500000.
        elif (self.resolution == self.ADC_RESOLUTIONS["14"]) or (
                self.resolution == self.ADC_RESOLUTIONS["15"]):
            dt = (timebase - 2.0) / 125000000.
        elif self.resolution == self.ADC_RESOLUTIONS["16"]:
            dt = (timebase - 3.0) / 62500000.
        return dt

    def _lowLevelSetAWGSimpleDeltaPhase(self, waveform, deltaPhase,
                                        offsetVoltage, pkToPk, indexMode,
                                        shots, triggerType, triggerSource):
        """Waveform should be an array of shorts."""
        waveformPtr = waveform.ctypes.data_as(POINTER(c_int16))

        m = self.lib.ps5000aSetSigGenArbitrary(
            c_int16(self.handle),
            c_uint32(int(offsetVoltage * 1E6)),  # offset voltage in microvolts
            c_uint32(int(pkToPk * 1E6)),         # pkToPk in microvolts
            c_uint32(int(deltaPhase)),           # startDeltaPhase
            c_uint32(int(deltaPhase)),           # stopDeltaPhase
            c_uint32(0),                         # deltaPhaseIncrement
            c_uint32(0),                         # dwellCount
            waveformPtr,                         # arbitraryWaveform
            c_int32(len(waveform)),              # arbitraryWaveformSize
            c_enum(0),                           # sweepType for deltaPhase
            c_enum(0),            # operation (adding random noise and whatnot)
            c_enum(indexMode),                   # single, dual, quad
            c_uint32(shots),
            c_uint32(0),                         # sweeps
            c_uint32(triggerType),
            c_uint32(triggerSource),
            c_int16(0))                          # extInThreshold
        self.checkResult(m)

    def _lowLevelSetDataBuffer(self, channel, data, downSampleMode,
                               segmentIndex):
        """Set the data buffer.

        Be sure to call _lowLevelClearDataBuffer
        when you are done with the data array
        or else subsequent calls to GetValue will still use the same array.
        """
        dataPtr = data.ctypes.data_as(POINTER(c_int16))
        numSamples = len(data)

        m = self.lib.ps5000aSetDataBuffer(c_int16(self.handle),
                                          c_enum(channel),
                                          dataPtr, c_int32(numSamples),
                                          c_uint32(segmentIndex),
                                          c_enum(downSampleMode))
        self.checkResult(m)

    def _lowLevelSetDataBufferBulk(self, channel, data, segmentIndex,
                                   downSampleMode):
        """Just calls setDataBuffer with argument order changed.

        For compatibility with current picobase.py.
        """
        self._lowLevelSetDataBuffer(channel,
                                    data,
                                    downSampleMode,
                                    segmentIndex)

    def _lowLevelClearDataBuffer(self, channel, segmentIndex):
        m = self.lib.ps5000aSetDataBuffer(c_int16(self.handle),
                                          c_enum(channel),
                                          c_void_p(), c_uint32(0),
                                          c_uint32(segmentIndex),
                                          c_enum(0))
        self.checkResult(m)

    def _lowLevelGetValues(self, numSamples, startIndex, downSampleRatio,
                           downSampleMode, segmentIndex):
        numSamplesReturned = c_uint32()
        numSamplesReturned.value = numSamples
        overflow = c_int16()
        m = self.lib.ps5000aGetValues(
            c_int16(self.handle), c_uint32(startIndex),
            byref(numSamplesReturned), c_uint32(downSampleRatio),
            c_enum(downSampleMode), c_uint32(segmentIndex),
            byref(overflow))
        self.checkResult(m)
        return (numSamplesReturned.value, overflow.value)

    def _lowLevelSetSigGenBuiltInSimple(self, offsetVoltage, pkToPk, waveType,
                                        frequency, shots, triggerType,
                                        triggerSource, stopFreq, increment,
                                        dwellTime, sweepType, numSweeps):
        # TODO, I just noticed that V2 exists
        # Maybe change to V2 in the future

        if stopFreq is None:
            stopFreq = frequency

        m = self.lib.ps5000aSetSigGenBuiltIn(
            c_int16(self.handle),
            c_int32(int(offsetVoltage * 1000000)),
            c_int32(int(pkToPk * 1000000)),
            c_int16(waveType),
            c_float(frequency), c_float(stopFreq),
            c_float(increment), c_float(dwellTime),
            c_enum(sweepType), c_enum(0),
            c_uint32(shots), c_uint32(numSweeps),
            c_enum(triggerType), c_enum(triggerSource),
            c_int16(0))
        self.checkResult(m)

    def _lowLevelSetDeviceResolution(self, resolution):
        self.resolution = resolution
        m = self.lib.ps5000aSetDeviceResolution(
            c_int16(self.handle),
            c_enum(resolution))
        self.checkResult(m)

    def _lowLevelChangePowerSource(self, powerstate):
        m = self.lib.ps5000aChangePowerSource(
            c_int16(self.handle),
            c_enum(powerstate))
        self.checkResult(m)

    # Morgan's additions
    def _lowLevelGetValuesBulk(self, numSamples, fromSegment, toSegment,
                               downSampleRatio, downSampleMode, overflow):
        """Copy data from several memory segments at once."""
        overflowPoint = overflow.ctypes.data_as(POINTER(c_int16))
        m = self.lib.ps5000aGetValuesBulk(
            c_int16(self.handle),
            byref(c_int32(numSamples)),
            c_int32(fromSegment),
            c_int32(toSegment),
            c_int32(downSampleRatio),
            c_enum(downSampleMode),
            overflowPoint
            )
        self.checkResult(m)

    def _lowLevelSetNoOfCaptures(self, numCaptures):
        m = self.lib.ps5000aSetNoOfCaptures(
            c_int16(self.handle),
            c_uint32(numCaptures))
        self.checkResult(m)

    def _lowLevelMemorySegments(self, numSegments):
        maxSamples = c_int32()
        m = self.lib.ps5000aMemorySegments(
            c_int16(self.handle), c_uint32(numSegments), byref(maxSamples))
        self.checkResult(m)
        return maxSamples.value

    def _lowLevelGetValuesTriggerTimeOffsetBulk(self, fromSegment, toSegment):
        """Supposedly gets the trigger times for a bunch of segments at once.

        For block mode.
        Can't get it to work yet, however.
        """
        import numpy as np

        nSegments = toSegment - fromSegment + 1
        # time = c_int64()
        times = np.ascontiguousarray(
            np.zeros(nSegments, dtype=np.int64)
            )
        timeUnits = np.ascontiguousarray(
            np.zeros(nSegments, dtype=np.int32)
            )

        m = self.lib.ps5000aGetValuesTriggerTimeOffsetBulk64(
            c_int16(self.handle),
            times.ctypes.data_as(POINTER(c_int64)),
            timeUnits.ctypes.data_as(POINTER(c_enum)),
            c_uint32(fromSegment),
            c_uint32(toSegment)
            )
        self.checkResult(m)
        # timeUnits=np.array([self.TIME_UNITS[tu] for tu in timeUnits])
        return times, timeUnits



"""
Test the garbage collection if we pass the picoscope object.

By: Mark Harfouche

"""

from __future__ import division
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

from picoscope import ps6000

# import the garbage collection interface
import gc

if __name__ == "__main__":
    ps = ps6000.PS6000()

    print("Found the following picoscope:")
    print("Serial: " + ps.getUnitInfo("BatchAndSerial"))

    pd = ps

    print("Copied the picoscope object. " +
          " the information of the copied object is:")
    print("Serial: " + pd.getUnitInfo("BatchAndSerial"))

    print("\n\n\n")

    print("Using both objects")
    ps.setChannel('A')
    pd.setChannel('B')

    del ps

    print("Deleting the original object and collecting garbage.")

    gc.collect()
    print("Copied object still works:")
    print("Serial: " + pd.getUnitInfo("BatchAndSerial"))

    pd.close()
    print("Now I closed the other object.")



"""
This is the low level driver file for a specific Picoscope.

By this, I mean if parameters want to get passed as strings, they should be
handled by PSBase
All functions here should take things as close to integers as possible, the
only exception here is for array parameters. Array parameters should be passed
in a pythonic way through numpy since the PSBase class should not be aware of
the specifics behind how the clib is called.

The functions should not have any default values as these should be handled
by PSBase.
"""


# pico-python is Copyright (c) 2013-2014 By:
# Colin O'Flynn <coflynn@newae.com>
# Mark Harfouche <mark.harfouche@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from __future__ import division
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

# import math

import inspect

# to load the proper dll
import platform

# Do not import or use ill definied data types
# such as short int or long
# use the values specified in the h file
# float is always defined as 32 bits
# double is defined as 64 bits
from ctypes import byref, POINTER, create_string_buffer, c_float, \
    c_int16, c_int32, c_uint32, c_void_p
from ctypes import c_int32 as c_enum

from picoscope.picobase import _PicoscopeBase


class PS3000(_PicoscopeBase):
    """The following are low-level functions for the PS3000."""

    LIBNAME = "ps3000"

    NUM_CHANNELS = 2
    CHANNELS = {"A": 0, "B": 1, "MaxChannels": 2}

    THRESHOLD_TYPE = {"Rising": 0,
                      "Falling": 1}

    CHANNEL_RANGE = [
        {"rangeV": 20E-3, "apivalue": 1, "rangeStr": "20 mV"},
        {"rangeV": 50E-3, "apivalue": 2, "rangeStr": "50 mV"},
        {"rangeV": 100E-3, "apivalue": 3, "rangeStr": "100 mV"},
        {"rangeV": 200E-3, "apivalue": 4, "rangeStr": "200 mV"},
        {"rangeV": 500E-3, "apivalue": 5, "rangeStr": "500 mV"},
        {"rangeV": 1.0, "apivalue": 6, "rangeStr": "1 V"},
        {"rangeV": 2.0, "apivalue": 7, "rangeStr": "2 V"},
        {"rangeV": 5.0, "apivalue": 8, "rangeStr": "5 V"},
        {"rangeV": 10.0, "apivalue": 9, "rangeStr": "10 V"},
        {"rangeV": 20.0, "apivalue": 10, "rangeStr": "20 V"},
        ]

    CHANNEL_COUPLINGS = {"DC": 1, "AC": 0}

    # has_sig_gen = True
    WAVE_TYPES = {"Sine": 0, "Square": 1, "Triangle": 2,
                  "RampUp": 3, "RampDown": 4, "DCVoltage": 5}

    SWEEP_TYPES = {"Up": 0, "Down": 1, "UpDown": 2, "DownUp": 3}

    TIME_UNITS = {"FS": 0, "PS": 1, "NS": 2, "US": 3, "MS": 4, "S": 5}

    MAX_VALUE = 32767
    MIN_VALUE = -32767

    MAX_TIMEBASES = 19

    UNIT_INFO_TYPES = {"DriverVersion": 0x0,
                       "USBVersion": 0x1,
                       "HardwareVersion": 0x2,
                       "VariantInfo": 0x3,
                       "BatchAndSerial": 0x4,
                       "CalDate": 0x5,
                       "ErrorCode": 0x6,
                       "KernelVersion": 0x7}

    channelBuffersPtr = [c_void_p(), c_void_p()]
    channelBuffersLen = [0, 0]

    # SIGGEN_TRIGGER_TYPES = {"Rising": 0, "Falling": 1,
    #                         "GateHigh": 2, "GateLow": 3}
    # SIGGEN_TRIGGER_SOURCES = {"None": 0, "ScopeTrig": 1, "AuxIn": 2,
    #                           "ExtIn": 3, "SoftTrig": 4, "TriggerRaw": 5}

    def __init__(self, serialNumber=None, connect=True):
        """Load DLL etc."""
        if platform.system() == 'Linux':
            from ctypes import cdll
            self.lib = cdll.LoadLibrary("lib" + self.LIBNAME + ".so")
        elif platform.system() == 'Darwin':
            from picoscope.darwin_utils import LoadLibraryDarwin
            self.lib = LoadLibraryDarwin("lib" + self.LIBNAME + ".dylib")
        else:
            from ctypes import windll
            from ctypes.util import find_library
            self.lib = windll.LoadLibrary(
                find_library(str(self.LIBNAME + ".dll"))
            )

        super(PS3000, self).__init__(serialNumber, connect)

    def _lowLevelOpenUnit(self, sn):
        if sn is not None:
            raise ValueError("PS3000 Doesn't Support Open by Serial Number")

        m = self.lib.ps3000_open_unit()

        if m < 0:
            raise IOError("Failed to Find PS3000 Unit. " +
                          "Should you be using PS3000a driver?")

        self.handle = m
        self.suggested_time_units = self.TIME_UNITS["NS"]

    def _lowLevelCloseUnit(self):
        m = self.lib.ps3000_close_unit(c_int16(self.handle))
        self.checkResult(m)

    def _lowLevelSetChannel(self, chNum, enabled, coupling, VRange, VOffset,
                            BWLimited):
        m = self.lib.ps3000_set_channel(
            c_int16(self.handle), c_enum(chNum),
            c_int16(enabled), c_enum(coupling),
            c_enum(VRange))
        self.checkResult(m)

    def _lowLevelStop(self):
        m = self.lib.ps3000_stop(c_int16(self.handle))
        self.checkResult(m)

    def _lowLevelGetUnitInfo(self, info):
        s = create_string_buffer(256)

        m = self.lib.ps3000_get_unit_info(
            c_int16(self.handle), byref(s), c_int16(len(s)), c_enum(info))
        self.checkResult(m)

        # should this bee ascii instead?
        # I think they are equivalent...
        return s.value.decode('utf-8')

    def _lowLevelFlashLed(self, times):
        m = self.lib.ps3000_flash_led(c_int16(self.handle))
        self.checkResult(m)

    def _lowLevelSetSimpleTrigger(self, enabled, trigsrc, threshold_adc,
                                  direction, delay, timeout_ms):
        # TODO: Fix 'auto' which is where trigger occurs in block.
        # Delay is not used

        m = self.lib.ps3000_set_trigger(
            c_int16(self.handle), c_enum(trigsrc), c_int16(threshold_adc),
            c_enum(direction), c_int16(delay), c_int16(timeout_ms))
        self.checkResult(m)

    def _lowLevelRunBlock(self, numPreTrigSamples, numPostTrigSamples,
                          timebase, oversample, segmentIndex, callback,
                          pParameter):
        # NOT: Oversample is NOT used!
        # Note: callback and pParameter are not available.

        # TODO: Fix 'delay' which is where trigger occurs in block
        if numPreTrigSamples > 0:
            raise ValueError("numPreTrigSamples isn't supported on PS3000")

        timeIndisposedMs = c_int32()
        m = self.lib.ps3000_run_block(
            c_int16(self.handle), c_uint32(numPostTrigSamples),
            c_uint32(timebase), c_int16(1), byref(timeIndisposedMs))

        self.checkResult(m)
        return timeIndisposedMs.value

    def _lowLevelIsReady(self):
        ready = c_int16()
        ready = self.lib.ps3000_ready(c_int16(self.handle))
        if ready > 0:
            return True
        elif ready == 0:
            return False
        else:
            raise IOError("ps3000_ready returned %d" % ready.value)

    def _lowLevelGetTimebase(self, tb, noSamples, oversample, segmentIndex):
        """Return (timeIntervalSeconds, maxSamples)."""
        maxSamples = c_int32()
        time_interval = c_int32()
        time_units = c_int16()

        m = self.lib.ps3000_get_timebase(
            c_int16(self.handle), c_int16(tb), c_uint32(noSamples),
            byref(time_interval), byref(time_units), c_int16(1),
            byref(maxSamples))

        self.checkResult(m)

        self.suggested_time_units = time_units.value

        return (time_interval.value / 1.0E9, maxSamples.value)

    def getTimeBaseNum(self, sampleTimeS):
        """ps3000 doesn't seem to have published formula like other scopes."""
        time_interval = c_int32()
        timebases = [None] * self.MAX_TIMEBASES

        # Convert to nS
        sampleTimenS = sampleTimeS * 1E9

        tb = 0
        while tb < self.MAX_TIMEBASES:
            rv = self.lib.ps3000_get_timebase(
                c_int16(self.handle), c_int16(tb), c_uint32(512),
                byref(time_interval), c_void_p(), c_int16(1),  c_void_p())
            if rv != 0:
                timebases[tb] = time_interval.value

            tb += 1

        # Figure out closest option
        besterror = 1E99
        bestindx = 0
        for indx, val in enumerate(timebases):
            if val is not None:
                error = sampleTimenS - val
                if abs(error) < besterror:
                    besterror = abs(error)
                    bestindx = indx

        return bestindx

    def getTimestepFromTimebase(self, timebase):
        """Return timestep from Timebase."""
        time_interval = c_int32()
        m = self.lib.ps3000_get_timebase(
            c_int16(self.handle), c_int16(timebase), c_uint32(512),
            byref(time_interval), c_void_p(), c_int16(1),  c_void_p())
        self.checkResult(m)
        return (time_interval.value / 1.0E9)

    def _lowLevelSetDataBuffer(self, channel, data, downSampleMode,
                               segmentIndex):
        dataPtr = data.ctypes.data_as(POINTER(c_int16))
        numSamples = len(data)

        self.channelBuffersPtr[channel] = dataPtr
        self.channelBuffersLen[channel] = numSamples

    def _lowLevelClearDataBuffer(self, channel, segmentIndex):
        self.channelBuffersPtr[channel] = c_void_p()
        self.channelBuffersLen[channel] = 0

    def _lowLevelGetValues(self, numSamples, startIndex, downSampleRatio,
                           downSampleMode, segmentIndex):
        # TODO:
        # Check overflow in channelBuffersLen against numSamples,
        # but need to not raise error if channelBuffersPtr is void

        overflow = c_int16()
        rv = self.lib.ps3000_get_values(
            c_int16(self.handle),
            self.channelBuffersPtr[0],
            self.channelBuffersPtr[1],
            c_void_p(), c_void_p(),
            byref(overflow), c_int32(numSamples))

        self.checkResult(rv)
        return (rv, overflow.value)

    def _lowLevelSetSigGenBuiltInSimple(self, offsetVoltage, pkToPk, waveType,
                                        frequency, shots, triggerType,
                                        triggerSource, stopFreq, increment,
                                        dwellTime, sweepType, numSweeps):
        if stopFreq is None:
            stopFreq = frequency

        m = self.lib.ps3000_set_sig_gen_built_in(
            c_int16(self.handle),
            c_int32(int(offsetVoltage * 1000000)),
            c_int32(int(pkToPk * 1000000)),
            c_int16(waveType),
            c_float(frequency), c_float(stopFreq),
            c_float(increment), c_float(dwellTime), c_enum(sweepType),
            c_uint32(numSweeps))
        self.checkResult(m)

    def checkResult(self, ec):
        """Check result of function calls, raise exception if not 0."""
        # PS3000 differs from other drivers in that non-zero is good
        if ec == 0:
            raise IOError('Error calling %s' % (inspect.stack()[1][3]))

        return 0



"""
PS6000 AWG Demo.

By: Mark Harfouche

This is a demo of how to use AWG with the Picoscope 6000
It was tested with the PS6403B USB2.0 version

The AWG is connected to Channel A.
Nothing else is required

Warning, there seems to be a bug with AWG
see http://www.picotech.com/support/topic12969.html

"""
from __future__ import division
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import time
from picoscope import ps6000
import pylab as plt
import numpy as np

if __name__ == "__main__":
    print(__doc__)

    print("Attempting to open Picoscope 6000...")

    # see page 13 of the manual to understand how to work this beast
    ps = ps6000.PS6000()

    print("Found the following picoscope:")
    print(ps.getAllUnitInfo())

    waveform_desired_duration = 1E-3
    obs_duration = 3 * waveform_desired_duration
    sampling_interval = obs_duration / 4096

    (actualSamplingInterval, nSamples, maxSamples) = \
        ps.setSamplingInterval(sampling_interval, obs_duration)
    print("Sampling interval = %f ns" % (actualSamplingInterval * 1E9))
    print("Taking  samples = %d" % nSamples)
    print("Maximum samples = %d" % maxSamples)

    waveformAmplitude = 1.5
    waveformOffset = 0
    x = np.linspace(-1, 1, num=ps.AWGMaxSamples, endpoint=False)
    # generate an interesting looking waveform
    waveform = waveformOffset + (x / 2 + (x ** 2) / 2) * waveformAmplitude

    (waveform_duration, deltaPhase) = ps.setAWGSimple(
        waveform, waveform_desired_duration, offsetVoltage=0.0,
        indexMode="Dual", triggerSource='None')

    # the setChannel command will chose the next largest amplitude
    # BWLimited = 1 for 6402/6403, 2 for 6404, 0 for all
    channelRange = ps.setChannel('A', 'DC', waveformAmplitude, 0.0,
                                 enabled=True, BWLimited=False)

    print("Chosen channel range = %d" % channelRange)

    ps.setSimpleTrigger('A', 1.0, 'Falling', delay=0, timeout_ms=100,
                        enabled=True)
    # ps.setSimpleTrigger('TriggerAux', 0.0, 'Falling', delay=0,
    #                     timeout_ms=100, enabled=True)

    ps.runBlock()
    ps.waitReady()
    print("Waiting for awg to settle.")
    time.sleep(2.0)
    ps.runBlock()
    ps.waitReady()
    print("Done waiting for trigger")
    dataA = ps.getDataV('A', nSamples, returnOverflow=False)

    dataTimeAxis = np.arange(nSamples) * actualSamplingInterval

    ps.stop()
    ps.close()

    plt.ion()
    plt.figure()
    plt.hold(True)
    plt.plot(dataTimeAxis, dataA, label="Clock")
    plt.grid(True, which='major')
    plt.title("Picoscope 6000 waveforms")
    plt.ylabel("Voltage (V)")
    plt.xlabel("Time (ms)")
    plt.legend()
    plt.show()



from setuptools import setup
import versioneer


with open('README.md') as readme_file:
    long_description = readme_file.read()


setup(
    name='picoscope',
    version=versioneer.get_version(),
    description="Unofficial python wrapper for the PicoScope devices.",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author="Colin O'Flynn, Mark Harfouche",
    author_email='coflynn@newae.com, mark.harfouche@gmail.com',
    license='BSD',
    url='https://github.com/colinoflynn/pico-python/',
    packages=['picoscope'],
    # See https://PyPI.python.org/PyPI?%3Aaction=list_classifiers
    classifiers=['Development Status :: 4 - Beta',
                 'Intended Audience :: Developers',
                 'Topic :: Software Development :: Libraries',
                 'Topic :: System :: Hardware',
                 'Topic :: Scientific/Engineering',
                 'License :: OSI Approved :: BSD License',
                 'Programming Language :: Python :: 2.7',
                 'Programming Language :: Python :: 3',
                 ],

    # What does your project relate to?
    keywords='picoscope peripherals hardware oscilloscope ATE',
    install_requires=['numpy'],
    cmdclass=versioneer.get_cmdclass()
)



# This is the instrument-specific file for the PS6000 series of instruments.
#
# pico-python is Copyright (c) 2013-2014 By:
# Colin O'Flynn <coflynn@newae.com>
# Mark Harfouche <mark.harfouche@gmail.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# 1. Redistributions of source code must retain the above copyright notice,
#    this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

"""
This is the low level driver file for a specific Picoscope.

By this, I mean if parameters want to get passed as strings, they should be
handled by PSBase
All functions here should take things as close to integers as possible, the
only exception here is for array parameters. Array parameters should be passed
in a pythonic way through numpy since the PSBase class should not be aware of
the specifics behind how the clib is called.

The functions should not have any default values as these should be handled
by PSBase.

"""

# 3.0 compatibility
# see http://docs.python.org/2/library/__future__.html
from __future__ import division
from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

import math

# to load the proper dll
import platform

# Do not import or use ill definied data types
# such as short int or long
# use the values specified in the h file
# float is always defined as 32 bits
# double is defined as 64 bits
from ctypes import byref, POINTER, create_string_buffer, c_float, \
    c_int8, c_int16, c_int32, c_uint32, c_uint64, c_void_p
from ctypes import c_int32 as c_enum

from picoscope.picobase import _PicoscopeBase


class PS6000(_PicoscopeBase):
    """The following are low-level functions for the PS6000."""

    LIBNAME = "ps6000"

    MAX_VALUE = 32764
    MIN_VALUE = -32764

    # EXT/AUX seems to have an imput impedence of 50 ohm (PS6403B)
    EXT_MAX_VALUE = 32767
    EXT_MIN_VALUE = -32767
    EXT_RANGE_VOLTS = 1

    # I don't think that the 50V range is allowed, but I left it there anyway
    # The 10V and 20V ranges are only allowed in high impedence modes
    CHANNEL_RANGE = [{"rangeV": 20E-3,  "apivalue": 1, "rangeStr": "20 mV"},
                     {"rangeV": 50E-3,  "apivalue": 2, "rangeStr": "50 mV"},
                     {"rangeV": 100E-3, "apivalue": 3, "rangeStr": "100 mV"},
                     {"rangeV": 200E-3, "apivalue": 4, "rangeStr": "200 mV"},
                     {"rangeV": 500E-3, "apivalue": 5, "rangeStr": "500 mV"},
                     {"rangeV": 1.0,    "apivalue": 6, "rangeStr": "1 V"},
                     {"rangeV": 2.0,    "apivalue": 7, "rangeStr": "2 V"},
                     {"rangeV": 5.0,    "apivalue": 8, "rangeStr": "5 V"},
                     {"rangeV": 10.0,   "apivalue": 9, "rangeStr": "10 V"},
                     {"rangeV": 20.0,   "apivalue": 10, "rangeStr": "20 V"},
                     {"rangeV": 50.0,   "apivalue": 11, "rangeStr": "50 V"},
                     ]

    NUM_CHANNELS = 4
    CHANNELS = {"A": 0, "B": 1, "C": 2, "D": 3,
                "External": 4, "MaxChannels": 4, "TriggerAux": 5}

    CHANNEL_COUPLINGS = {"DC50": 2, "DC": 1, "AC": 0}

    WAVE_TYPES = {"Sine": 0, "Square": 1, "Triangle": 2,
                  "RampUp": 3, "RampDown": 4,
                  "Sinc": 5, "Gaussian": 6, "HalfSine": 7, "DCVoltage": 8,
                  "WhiteNoise": 9}

    SWEEP_TYPES = {"Up": 0, "Down": 1, "UpDown": 2, "DownUp": 3}

    SIGGEN_TRIGGER_TYPES = {"Rising": 0, "Falling": 1,
                            "GateHigh": 2, "GateLow": 3}
    SIGGEN_TRIGGER_SOURCES = {"None": 0, "ScopeTrig": 1, "AuxIn": 2,
                              "ExtIn": 3, "SoftTrig": 4, "TriggerRaw": 5}

    # This is actually different depending on the AB/CD models
    # I wonder how we could detect the difference between the oscilloscopes
    # I believe we can obtain this information from the setInfo function
    # by readign the hardware version
    # for the PS6403B version, the hardware version is "1 1",
    # an other possibility is that the PS6403B shows up as 6403 when using
    # VARIANT_INFO and others show up as PS6403X where X = A,C or D

    AWGPhaseAccumulatorSize = 32
    AWGBufferAddressWidth = 14
    AWGMaxSamples = 2 ** AWGBufferAddressWidth

    AWGDACInterval = 5E-9  # in seconds
    AWGDACFrequency = 1 / AWGDACInterval

    # Note this is NOT what is written in the Programming guide as of version
    # 10_5_0_28
    # This issue was acknowledged in this thread
    # http://www.picotech.com/support/topic13217.html
    AWGMaxVal = 0x0FFF
    AWGMinVal = 0x0000

    AWG_INDEX_MODES = {"Single": 0, "Dual": 1, "Quad": 2}

    def __init__(self, serialNumber=None, connect=True):
        """Load DLLs."""
        if platform.system() == 'Linux':
            from ctypes import cdll
            # ok I don't know what is wrong with my installer,
            # but I need to include .so.2
            self.lib = cdll.LoadLibrary("lib" + self.LIBNAME + ".so.2")
        elif platform.system() == 'Darwin':
            from picoscope.darwin_utils import LoadLibraryDarwin
            self.lib = LoadLibraryDarwin("lib" + self.LIBNAME + ".dylib")
        else:
            from ctypes import windll
            from ctypes.util import find_library
            self.lib = windll.LoadLibrary(
                find_library(str(self.LIBNAME + ".dll"))
            )

        super(PS6000, self).__init__(serialNumber, connect)

    def _lowLevelOpenUnit(self, serialNumber):
        c_handle = c_int16()
        if serialNumber is not None:
            serialNumberStr = create_string_buffer(bytes(serialNumber,
                                                         encoding='utf-8'))
        else:
            serialNumberStr = None
        # Passing None is the same as passing NULL
        m = self.lib.ps6000OpenUnit(byref(c_handle), serialNumberStr)
        self.checkResult(m)
        self.handle = c_handle.value

    def _lowLevelOpenUnitAsync(self, serialNumber):
        c_status = c_int16()
        if serialNumber is not None:
            serialNumberStr = create_string_buffer(bytes(serialNumber,
                                                         encoding='utf-8'))
        else:
            serialNumberStr = None
        # Passing None is the same as passing NULL
        m = self.lib.ps6000OpenUnitAsync(byref(c_status), serialNumberStr)
        self.checkResult(m)

        return c_status.value

    def _lowLevelOpenUnitProgress(self):
        complete = c_int16()
        progressPercent = c_int16()
        handle = c_int16()

        m = self.lib.ps6000OpenUnitProgress(byref(handle),
                                            byref(progressPercent),
                                            byref(complete))
        self.checkResult(m)

        if complete.value != 0:
            self.handle = handle.value

        # if we only wanted to return one value, we could do somethign like
        # progressPercent = progressPercent * (1 - 0.1 * complete)
        return (progressPercent.value, complete.value)

    def _lowLevelCloseUnit(self):
        m = self.lib.ps6000CloseUnit(c_int16(self.handle))
        self.checkResult(m)

    def _lowLevelEnumerateUnits(self):
        count = c_int16(0)
        serials = c_int8(0)
        serialLth = c_int16(0)
        m = self.lib.ps6000EnumerateUnits(byref(count), byref(serials),
                                          byref(serialLth))
        self.checkResult(m)
        # a serial number is rouhgly 8 characters
        # an extra character for the comma
        # and an extra one for the space after the comma?
        # the extra two also work for the null termination
        serialLth = c_int16(count.value * (8 + 2))
        serials = create_string_buffer(serialLth.value + 1)

        m = self.lib.ps6000EnumerateUnits(byref(count), serials,
                                          byref(serialLth))
        self.checkResult(m)

        serialList = str(serials.value.decode('utf-8')).split(',')

        serialList = [x.strip() for x in serialList]

        return serialList

    def _lowLevelSetChannel(self, chNum, enabled, coupling, VRange, VOffset,
                            BWLimited):
        m = self.lib.ps6000SetChannel(c_int16(self.handle), c_enum(chNum),
                                      c_int16(enabled), c_enum(coupling),
                                      c_enum(VRange), c_float(VOffset),
                                      c_enum(BWLimited))  # 2 for PS6404
        self.checkResult(m)

    def _lowLevelStop(self):
        m = self.lib.ps6000Stop(c_int16(self.handle))
        self.checkResult(m)

    def _lowLevelGetUnitInfo(self, info):
        s = create_string_buffer(256)
        requiredSize = c_int16(0)

        m = self.lib.ps6000GetUnitInfo(c_int16(self.handle), byref(s),
                                       c_int16(len(s)), byref(requiredSize),
                                       c_enum(info))
        self.checkResult(m)
        if requiredSize.value > len(s):
            s = create_string_buffer(requiredSize.value + 1)
            m = self.lib.ps6000GetUnitInfo(c_int16(self.handle), byref(s),
                                           c_int16(len(s)),
                                           byref(requiredSize), c_enum(info))
            self.checkResult(m)

        # should this bee ascii instead?
        # I think they are equivalent...
        return s.value.decode('utf-8')

    def _lowLevelFlashLed(self, times):
        m = self.lib.ps6000FlashLed(c_int16(self.handle), c_int16(times))
        self.checkResult(m)

    def _lowLevelSetSimpleTrigger(self, enabled, trigsrc, threshold_adc,
                                  direction, delay, timeout_ms):
        m = self.lib.ps6000SetSimpleTrigger(
            c_int16(self.handle), c_int16(enabled),
            c_enum(trigsrc), c_int16(threshold_adc),
            c_enum(direction), c_uint32(delay), c_int16(timeout_ms))
        self.checkResult(m)

    def _lowLevelRunBlock(self, numPreTrigSamples, numPostTrigSamples,
                          timebase, oversample, segmentIndex, callback,
                          pParameter):
        timeIndisposedMs = c_int32()
        m = self.lib.ps6000RunBlock(
            c_int16(self.handle), c_uint32(numPreTrigSamples),
            c_uint32(numPostTrigSamples), c_uint32(timebase),
            c_int16(oversample), byref(timeIndisposedMs),
            c_uint32(segmentIndex), c_void_p(), c_void_p())
        # According to the documentation, 'callback, pParameter' should work
        # instead of the last two c_void_p parameters.
        self.checkResult(m)
        return timeIndisposedMs.value

    def _lowLevelIsReady(self):
        ready = c_int16()
        m = self.lib.ps6000IsReady(c_int16(self.handle), byref(ready))
        self.checkResult(m)
        if ready.value:
            return True
        else:
            return False

    def _lowLevelGetTimebase(self, tb, noSamples, oversample, segmentIndex):
        """Return (timeIntervalSeconds, maxSamples)."""
        maxSamples = c_int32()
        sampleRate = c_float()

        m = self.lib.ps6000GetTimebase2(c_int16(self.handle), c_uint32(tb),
                                        c_uint32(noSamples), byref(sampleRate),
                                        c_int16(oversample), byref(maxSamples),
                                        c_uint32(segmentIndex))
        self.checkResult(m)

        return (sampleRate.value / 1.0E9, maxSamples.value)

    def getTimeBaseNum(self, sampleTimeS):
        """Return sample time in seconds to timebase as int for API calls."""
        maxSampleTime = (((2 ** 32 - 1) - 4) / 156250000)

        if sampleTimeS < 6.4E-9:
            timebase = math.floor(math.log(sampleTimeS * 5E9, 2))
            timebase = max(timebase, 0)
        else:
            # Otherwise in range 2^32-1
            if sampleTimeS > maxSampleTime:
                sampleTimeS = maxSampleTime

            timebase = math.floor((sampleTimeS * 156250000) + 4)

        # is this cast needed?
        timebase = int(timebase)
        return timebase

    def getTimestepFromTimebase(self, timebase):
        """Return timebase to sampletime as seconds."""
        if timebase < 5:
            dt = 2. ** timebase / 5E9
        else:
            dt = (timebase - 4.) / 156250000.
        return dt

    def _lowLevelSetAWGSimpleDeltaPhase(self, waveform, deltaPhase,
                                        offsetVoltage, pkToPk, indexMode,
                                        shots, triggerType, triggerSource):
        """Waveform should be an array of shorts."""
        waveformPtr = waveform.ctypes.data_as(POINTER(c_int16))

        m = self.lib.ps6000SetSigGenArbitrary(
            c_int16(self.handle),
            c_uint32(int(offsetVoltage * 1E6)),  # offset voltage in microvolts
            c_uint32(int(pkToPk * 1E6)),         # pkToPk in microvolts
            c_uint32(int(deltaPhase)),           # startDeltaPhase
            c_uint32(int(deltaPhase)),           # stopDeltaPhase
            c_uint32(0),                         # deltaPhaseIncrement
            c_uint32(0),                         # dwellCount
            waveformPtr,                         # arbitraryWaveform
            c_int32(len(waveform)),              # arbitraryWaveformSize
            c_enum(0),                           # sweepType for deltaPhase
            c_enum(0),            # operation (adding random noise and whatnot)
            c_enum(indexMode),                   # single, dual, quad
            c_uint32(shots),
            c_uint32(0),                         # sweeps
            c_uint32(triggerType),
            c_uint32(triggerSource),
            c_int16(0))                          # extInThreshold
        self.checkResult(m)

    def _lowLevelSetDataBuffer(self, channel, data, downSampleMode,
                               segmentIndex):
        """Set the data buffer.

        Be sure to call _lowLevelClearDataBuffer
        when you are done with the data array
        or else subsequent calls to GetValue will still use the same array.

        segmentIndex is unused, but required by other versions of the API
        (eg PS5000a)
        """
        dataPtr = data.ctypes.data_as(POINTER(c_int16))
        numSamples = len(data)

        m = self.lib.ps6000SetDataBuffer(c_int16(self.handle), c_enum(channel),
                                         dataPtr, c_uint32(numSamples),
                                         c_enum(downSampleMode))
        self.checkResult(m)

    def _lowLevelClearDataBuffer(self, channel, segmentIndex):
        m = self.lib.ps6000SetDataBuffer(c_int16(self.handle), c_enum(channel),
                                         c_void_p(), c_uint32(0), c_enum(0))
        self.checkResult(m)

    def _lowLevelGetValues(self, numSamples, startIndex, downSampleRatio,
                           downSampleMode, segmentIndex):
        numSamplesReturned = c_uint32()
        numSamplesReturned.value = numSamples
        overflow = c_int16()
        m = self.lib.ps6000GetValues(
            c_int16(self.handle), c_uint32(startIndex),
            byref(numSamplesReturned), c_uint32(downSampleRatio),
            c_enum(downSampleMode), c_uint32(segmentIndex),
            byref(overflow))
        self.checkResult(m)
        return (numSamplesReturned.value, overflow.value)

    def _lowLevelSetSigGenBuiltInSimple(self, offsetVoltage, pkToPk, waveType,
                                        frequency, shots, triggerType,
                                        triggerSource, stopFreq, increment,
                                        dwellTime, sweepType, numSweeps):
        # TODO, I just noticed that V2 exists
        # Maybe change to V2 in the future

        if stopFreq is None:
            stopFreq = frequency

        m = self.lib.ps6000SetSigGenBuiltIn(
            c_int16(self.handle),
            c_int32(int(offsetVoltage * 1000000)),
            c_int32(int(pkToPk * 1000000)),
            c_int16(waveType),
            c_float(frequency), c_float(stopFreq),
            c_float(increment), c_float(dwellTime),
            c_enum(sweepType), c_enum(0),
            c_uint32(shots), c_uint32(numSweeps),
            c_enum(triggerType), c_enum(triggerSource),
            c_int16(0))
        self.checkResult(m)

    ####################################################################
    # Untested functions below                                         #
    #                                                                  #
    ####################################################################
    def _lowLevelGetAnalogueOffset(self, range, coupling):
        # TODO, populate properties with this function
        maximumVoltage = c_float()
        minimumVoltage = c_float()

        m = self.lib.ps6000GetAnalogueOffset(
            c_int16(self.handle), c_enum(range), c_enum(coupling),
            byref(maximumVoltage), byref(minimumVoltage))
        self.checkResult(m)

        return (maximumVoltage.value, minimumVoltage.value)

    def _lowLevelGetMaxDownSampleRatio(self, noOfUnaggregatedSamples,
                                       downSampleRatioMode, segmentIndex):
        maxDownSampleRatio = c_uint32()

        m = self.lib.ps6000GetMaxDownSampleRatio(
            c_int16(self.handle), c_uint32(noOfUnaggregatedSamples),
            byref(maxDownSampleRatio),
            c_enum(downSampleRatioMode), c_uint32(segmentIndex))
        self.checkResult(m)

        return maxDownSampleRatio.value

    def _lowLevelGetNoOfCaptures(self):
        nCaptures = c_uint32()

        m = self.lib.ps6000GetNoOfCaptures(c_int16(self.handle),
                                           byref(nCaptures))
        self.checkResult(m)

        return nCaptures.value

    def _lowLevelGetTriggerTimeOffset(self, segmentIndex):
        time = c_uint64()
        timeUnits = c_enum()

        m = self.lib.ps6000GetTriggerTimeOffset64(
            c_int16(self.handle), byref(time),
            byref(timeUnits), c_uint32(segmentIndex))
        self.checkResult(m)

        if timeUnits.value == 0:    # PS6000_FS
            return time.value * 1E-15
        elif timeUnits.value == 1:  # PS6000_PS
            return time.value * 1E-12
        elif timeUnits.value == 2:  # PS6000_NS
            return time.value * 1E-9
        elif timeUnits.value == 3:  # PS6000_US
            return time.value * 1E-6
        elif timeUnits.value == 4:  # PS6000_MS
            return time.value * 1E-3
        elif timeUnits.value == 5:  # PS6000_S
            return time.value * 1E0
        else:
            raise TypeError("Unknown timeUnits %d" % timeUnits.value)

    def _lowLevelMemorySegments(self, nSegments):
        nMaxSamples = c_uint32()

        m = self.lib.ps6000MemorySegments(c_int16(self.handle),
                                          c_uint32(nSegments),
                                          byref(nMaxSamples))
        self.checkResult(m)

        return nMaxSamples.value

    def _lowLevelSetDataBuffers(self, channel, bufferMax, bufferMin,
                                downSampleRatioMode):
        bufferMaxPtr = bufferMax.ctypes.data_as(POINTER(c_int16))
        bufferMinPtr = bufferMin.ctypes.data_as(POINTER(c_int16))
        bufferLth = len(bufferMax)

        m = self.lib.ps6000SetDataBuffers(c_int16(self.handle),
                                          c_enum(channel),
                                          bufferMaxPtr, bufferMinPtr,
                                          c_uint32(bufferLth),
                                          c_enum(downSampleRatioMode))
        self.checkResult(m)

    def _lowLevelClearDataBuffers(self, channel):
        m = self.lib.ps6000SetDataBuffers(
            c_int16(self.handle), c_enum(channel),
            c_void_p(), c_void_p(), c_uint32(0), c_enum(0))
        self.checkResult(m)

    # Bulk values.
    # These would be nice, but the user would have to provide us
    # with an array.
    # we would have to make sure that it is contiguous amonts other things
    def _lowLevelGetValuesBulk(self,
                               numSamples, fromSegmentIndex, toSegmentIndex,
                               downSampleRatio, downSampleRatioMode,
                               overflow):
        noOfSamples = c_uint32(numSamples)

        m = self.lib.ps6000GetValuesBulk(
            c_int16(self.handle),
            byref(noOfSamples),
            c_uint32(fromSegmentIndex), c_uint32(toSegmentIndex),
            c_uint32(downSampleRatio), c_enum(downSampleRatioMode),
            overflow.ctypes.data_as(POINTER(c_int16))
            )
        self.checkResult(m)
        return noOfSamples.value

    def _lowLevelSetDataBufferBulk(self, channel, buffer, waveform,
                                   downSampleRatioMode):
        bufferPtr = buffer.ctypes.data_as(POINTER(c_int16))
        bufferLth = len(buffer)

        m = self.lib.ps6000SetDataBufferBulk(
            c_int16(self.handle),
            c_enum(channel), bufferPtr, c_uint32(bufferLth),
            c_uint32(waveform), c_enum(downSampleRatioMode))
        self.checkResult(m)

    def _lowLevelSetDataBuffersBulk(self, channel, bufferMax, bufferMin,
                                    waveform, downSampleRatioMode):
        bufferMaxPtr = bufferMax.ctypes.data_as(POINTER(c_int16))
        bufferMinPtr = bufferMin.ctypes.data_as(POINTER(c_int16))

        bufferLth = len(bufferMax)

        m = self.lib.ps6000SetDataBuffersBulk(
            c_int16(self.handle), c_enum(channel),
            bufferMaxPtr, bufferMinPtr, c_uint32(bufferLth),
            c_uint32(waveform), c_enum(downSampleRatioMode))
        self.checkResult(m)

    def _lowLevelSetNoOfCaptures(self, nCaptures):
        m = self.lib.ps6000SetNoOfCaptures(c_int16(self.handle),
                                           c_uint32(nCaptures))
        self.checkResult(m)

    # ETS Functions
    def _lowLevelSetEts():
        pass

    def _lowLevelSetEtsTimeBuffer():
        pass

    def _lowLevelSetEtsTimeBuffers():
        pass

    def _lowLevelSetExternalClock():
        pass

    # Complicated triggering
    # need to understand structs for this one to work
    def _lowLevelIsTriggerOrPulseWidthQualifierEnabled():
        pass

    def _lowLevelGetValuesTriggerTimeOffsetBulk():
        pass

    def _lowLevelSetTriggerChannelConditions():
        pass

    def _lowLevelSetTriggerChannelDirections():
        pass

    def _lowLevelSetTriggerChannelProperties():
        pass

    def _lowLevelSetPulseWidthQualifier():
        pass

    def _lowLevelSetTriggerDelay():
        pass

    # Async functions
    # would be nice, but we would have to learn to implement callbacks
    def _lowLevelGetValuesAsync():
        pass

    def _lowLevelGetValuesBulkAsync():
        pass

    # overlapped functions
    def _lowLevelGetValuesOverlapped():
        pass

    def _lowLevelGetValuesOverlappedBulk():
        pass

    # Streaming related functions
    def _lowLevelGetStreamingLatestValues():
        pass

    def _lowLevelNoOfStreamingValues(self):
        noOfValues = c_uint32()

        m = self.lib.ps6000NoOfStreamingValues(c_int16(self.handle),
                                               byref(noOfValues))
        self.checkResult(m)

        return noOfValues.value

    def _lowLevelRunStreaming():
        pass

    def _lowLevelStreamingReady():
        pass



# -*- coding: utf-8
# Example by Colin O'Flynn
#
import time
import numpy as np
from picoscope import ps5000a

from matplotlib.mlab import find


class freqMeasure():
    def __init__(self):
        self.ps = ps5000a.PS5000a(connect=False)

    def openScope(self):
        self.ps.open()

        self.ps.setChannel("A", coupling="DC", VRange=5.0, probeAttenuation=10)
        self.ps.setChannel("B", enabled=False)
        self.ps.setChannel("C", enabled=False)
        self.ps.setChannel("D", enabled=False)
        res = self.ps.setSamplingFrequency(1000E6, 50000)
        self.sampleRate = res[0]
        print("Sampling @ %f MHz, %d samples" % (res[0]/1E6, res[1]))

        # Use external trigger to mark when we sample
        self.ps.setSimpleTrigger(trigSrc="External", threshold_V=0.150,
                                 timeout_ms=5000)

    def closeScope(self):
        self.ps.close()

    def armMeasure(self):
        self.ps.runBlock()

    def freq_from_crossings(self, sig):
        """Estimate frequency by counting zero crossings"""
        # From https://gist.github.com/endolith/255291:

        fs = self.sampleRate

        # Find all indices right before a rising-edge zero crossing
        indices = find((sig[1:] >= 0) & (sig[:-1] < 0))
        # More accurate, using linear interpolation to find intersample
        # zero-crossings (Measures 1000.000129 Hz for 1000 Hz, for instance)
        crossings = [i - sig[i] / (sig[i+1] - sig[i]) for i in indices]
        # Some other interpolation based on neighboring points might be better.
        # Spline, cubic, whatever
        return fs / np.mean(np.diff(crossings))

    def measure(self):
        print("Waiting for trigger")
        while not self.ps.isReady():
            time.sleep(0.01)
        print("Sampling Done")
        data = self.ps.getDataV("A", 50000)

        data = data - np.mean(data)
        freq = self.freq_from_crossings(data)

        print(freq)


if __name__ == "__main__":
    fm = freqMeasure()
    fm.openScope()

    try:
        while 1:
            fm.armMeasure()
            fm.measure()
    except KeyboardInterrupt:
        pass

    fm.closeScope()



from picoscope import ps4000a
import matplotlib.pyplot as plt
import time

ps = ps4000a.PS4000a()


def callbackFunction(handle, status, pParameter=None):
    """This function is executed once the block is ready."""
    if status == 0:
        print("Block is ready and can be read.")
        data = ps.getDataV('A')

        print("data transferred")
        plt.plot(data)
        plt.show()
    else:
        print("An error occurred.")


# Picoscope setup
ps.setChannel(channel="A", coupling="DC", VRange=1)
ps.setChannel(channel="B", enabled=False)
ps.setChannel(channel="C", enabled=False)
ps.setChannel(channel="D", enabled=False)

sample_interval = 100e-9  # 100 ns
sample_duration = 2e-3  # 1 ms
ps.setSamplingInterval(sample_interval, sample_duration)
ps.setSimpleTrigger("A", threshold_V=0.1, timeout_ms=1)

# Run a block
print("Run a block with a callback function")
ps.runBlock(callback=callbackFunction)

time.sleep(10)
ps.close()
print("end")



import numpy as np
import time

# collect 20% more samples than nominally necessary
SAMPLE_SAFETY_MARGIN = 1.25
LIGHTSPEED = 2.99e8


class RadarScope(object):
    """Interface to picoscope, connected to radar"""

    def __init__(self, ps, rotation_period=2.4, pulse_rate=2100,
                 range_resolution=10.0, max_range=1e3, video_chan="A",
                 video_voltage=[-0.1, 2], trigger_voltage=0.4,
                 heading_chan="B", heading_voltage=[1, -10], data_dir="."):
        '''
        Set up interface to PicoScope for radar data collection.

        Parameters
        ----------
        ps : ps3000a object
        rotation_period : float
            Nominal antenna rotation period.
        pulse_rate : int
            Nominal radar pulse rate.
        max_range : float
            range, in meters, to which samples will be recorderd
        range_resolution : float
            Nominal range resolution for radar data capturing. This is
            translated to the digital (time) sampling rate.  The actual
            physical range resolution is limited by the radar pulse length.
        video_chan, heading_chan : String
            Oscilloscope channels (e.g. "A", "B") connected to the radar's
            video and heading pulse signals.
        video_voltage, heading_voltage : List
            Lists containing the minimum and maximum DC voltages needed
            to capture the input signals
        trigger_voltage : float
            Trigger data collection when video rises past this threshold
        '''
        super(RadarScope, self).__init__()
        self.ps = ps
        self.rotation_period = rotation_period
        self.pulse_rate = pulse_rate
        self.max_range = max_range
        self.range_resolution = range_resolution
        self.video_chan = video_chan
        self.heading_chan = heading_chan
        self.video_voltage = video_voltage
        self.heading_voltage = heading_voltage
        self.n_captures = int(self.pulse_rate * rotation_period *
                              SAMPLE_SAFETY_MARGIN)
        self.sample_interval = 2 * self.range_resolution / LIGHTSPEED
        self.capture_duration = self.max_range * 2 / LIGHTSPEED
        self.data_dir = data_dir

        # set up oscilloscope channels, trigger, and sampling rate
        self.ps.setChannel(channel=self.video_chan, coupling="DC",
                           VRange=max([abs(x) for x in video_voltage]))
        self.ps.setChannel(channel=self.heading_chan, coupling="DC",
                           VRange=max([abs(x) for x in heading_voltage]))
        self.ps.setSamplingInterval(self.sample_interval,
                                    self.capture_duration)

        self.ps.setSimpleTrigger("A", threshold_V=trigger_voltage)
        self.max_samples_per_segment = self.ps.memorySegments(self.n_captures)
        self.samples_per_segment = int(self.capture_duration /
                                       self.sample_interval)
        self.ps.setNoOfCaptures(self.n_captures)

        self.video_buffer = np.zeros((self.n_captures,
                                      self.samples_per_segment),
                                     dtype=np.int16)
        self.heading_buffer = np.zeros((self.n_captures,
                                        self.samples_per_segment),
                                       dtype=np.int16)

    def run_sweep(self):
        '''
        Tells the PicoScope to run one rapid block of radar pulses
        according to the pre-specified settings.  Does not transfer the data
        collected from the PicoScope's memory to the computer.
        '''
        self.last_sweep_time = time.strftime("%Y%m%d_%H%M%S")
        self.ps.runBlock()

    def transfer_data(self, downsample_ratio=0, downsample_mode=0):
        '''
        Transfers stored data from the PicoScope's buffers to the computer's
        memory.
        '''
        t1 = time.time()
        ps.getDataRawBulk(channel=self.video_chan, data=self.video_buffer)
        ps.getDataRawBulk(channel=self.heading_chan, data=self.heading_buffer)
        print("Time to transfer data: " + str(time.time() - t1))

    def capture_sweep(self, downsample_ratio=0, downsample_mode=0):
        t1 = time.time()
        self.run_sweep()
        self.ps.waitReady()
        print("Time to capture data: " + str(time.time() - t1))
        self.transfer_data(downsample_ratio, downsample_mode)

    def zero_heading_indices(self, threshold=0):
        '''
        Returns the indices of the data blocks where the heading signal
        indicates that the antenna was at the zero point.
        '''
        pass

    def to_file(self, dir=None, echo=False):
        t1 = time.time()
        if dir is None:
            dir = self.data_dir
        filename = "sweep_" + self.last_sweep_time + ".swp"
        output_file = open(filename, "wb")
        output_file.write(self.video_buffer)
        output_file.write(self.heading_buffer)
        output_file.close()
        if echo:
            print(filename)
        print("Time to write data: " + str(time.time() - t1))

    def record(self, n_sweeps=1, minimum_interval=5):
        '''
        Capture n_sweeps sweeps, waiting no less than minimum_interval
        in between them.
        '''
        i = 0
        t1 = minimum_interval + 1
        while i < n_sweeps:
            while time.time() - t1 < minimum_interval:
                time.sleep(0)
            t1 = time.time()
            self.capture_sweep()
            self.to_file(echo=True)
            i += 1

    def disconnect(self):
        '''
        Closes the PicoScope API, releasing the oscilloscope.
        '''
        self.ps.close()


if __name__ == '__main__':
    import matplotlib.pyplot as plt
    # picoscope = reload(picoscope)
    from picoscope import ps3000a
    # ps3000a = reload(ps3000a)

    SERIAL_NUM = 'AR911/011\x00'
    ps = ps3000a.PS3000a(SERIAL_NUM)

    rscope = RadarScope(ps, max_range=1e3, range_resolution=5)

    rscope.capture_sweep()
    # rscope.to_file()
    # rscope.record(10, 5)

    plt.imshow(rscope.video_buffer, aspect="auto")
    plt.show()

    rscope.disconnect()
